@@ -25,6 +25,8 @@
 #include <limits.h>
 #include <stddef.h> /* for offsetof */
 
+#include <proj_api.h>
+
 #include "cavern.h"
 #include "commands.h"
 #include "datain.h"
@@ -177,12 +179,18 @@ static char *ucbuffer = NULL;
 /* read token */
 extern void
 get_token(void)
+{
+   skipblanks();
+   get_token_no_blanks();
+}
+
+extern void
+get_token_no_blanks(void)
 {
    int i = -1;
 
    s_zero(&buffer);
    osfree(ucbuffer);
-   skipblanks();
    while (isalpha(ch)) {
       s_catchar(&buffer, &buf_len, (char)ch);
       nextch();
@@ -196,7 +204,7 @@ get_token(void)
       ucbuffer[i] = toupper(buffer[i]);
    } while (buffer[i]);
 #if 0
-   printf("get_token() got “%s”\n", buffer);
+   printf("get_token_no_blanks() got “%s”\n", buffer);
 #endif
 }
 
@@ -242,7 +250,7 @@ match_tok(const sztok *tab, int tab_size)
 
 typedef enum {
    CMD_NULL = -1, CMD_ALIAS, CMD_BEGIN, CMD_CALIBRATE, CMD_CASE, CMD_COPYRIGHT,
-   CMD_DATA, CMD_DATE, CMD_DEFAULT, CMD_END, CMD_ENTRANCE, CMD_EQUATE,
+   CMD_CS, CMD_DATA, CMD_DATE, CMD_DEFAULT, CMD_END, CMD_ENTRANCE, CMD_EQUATE,
    CMD_EXPORT, CMD_FIX, CMD_FLAGS, CMD_INCLUDE, CMD_INFER, CMD_INSTRUMENT,
    CMD_PREFIX, CMD_REQUIRE, CMD_SD, CMD_SET, CMD_SOLVE,
    CMD_TEAM, CMD_TITLE, CMD_TRUNCATE, CMD_UNITS
@@ -254,6 +262,7 @@ static sztok cmd_tab[] = {
      {"CALIBRATE", CMD_CALIBRATE},
      {"CASE",      CMD_CASE},
      {"COPYRIGHT", CMD_COPYRIGHT},
+     {"CS",        CMD_CS},
      {"DATA",      CMD_DATA},
      {"DATE",      CMD_DATE},
 #ifndef NO_DEPRECATED
@@ -325,6 +334,9 @@ get_units(unsigned long qmask, bool percent_ok)
    if (((qmask & LEN_QMASK) && !TSTBIT(LEN_UMASK, units)) ||
        ((qmask & ANG_QMASK) && !TSTBIT(ANG_UMASK, units))) {
       file.lpos += strlen(buffer);
+      /* TRANSLATORS: Note: In English you talk about the *units* of a single
+       * measurement, but the correct term in other languages may be singular.
+       */
       compile_error_skip(-/*Invalid units “%s” for quantity*/37, buffer);
       return UNITS_NULL;
    }
@@ -441,8 +453,17 @@ cmd_set(void)
    if (mask == SPECIAL_ROOT) {
       if (root_depr_count < 5) {
 	 file.lpos += strlen(buffer);
+	 /* TRANSLATORS: Use of the ROOT character (which is "\" by default) is
+	  * deprecated, so this error would be generated by:
+	  * 
+	  * *equate \foo.7 1
+	  * 
+	  * If you're unsure what "deprecated" means, see:
+	  * http://en.wikipedia.org/wiki/Deprecation */
 	 compile_warning(-/*ROOT is deprecated*/25);
 	 if (++root_depr_count == 5)
+	     /* TRANSLATORS: If you're unsure what "deprecated" means, see:
+	      * http://en.wikipedia.org/wiki/Deprecation */
 	    compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
       }
    }
@@ -492,33 +513,35 @@ cmd_set(void)
 }
 
 static void
-check_reentry(prefix *tag)
+check_reentry(prefix *survey)
 {
    /* Don't try to check "*prefix \" or "*begin \" */
-   if (!tag->up) return;
-   if (TSTBIT(tag->sflags, SFLAGS_PREFIX_ENTERED)) {
-      if (tag->line != file.line ||
-	  strcmp(tag->filename, file.filename) != 0) {
-	 const char *filename_store = file.filename;
-	 unsigned int line_store = file.line;
-	 static int reenter_depr_count = 0;
+   if (!survey->up) return;
+   if (TSTBIT(survey->sflags, SFLAGS_PREFIX_ENTERED)) {
+      static int reenter_depr_count = 0;
 
-	 if (reenter_depr_count < 5) {
-	    compile_warning(/*Reentering an existing prefix level is deprecated*/29);
-	    if (++reenter_depr_count == 5)
-	       compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
-	 }
+      if (reenter_depr_count >= 5)
+	 return;
 
-	 file.filename = tag->filename;
-	 file.line = tag->line;
-	 compile_warning(/*Originally entered here*/30);
-	 file.filename = filename_store;
-	 file.line = line_store;
-      }
+      /* TRANSLATORS: e.g.
+       *
+       * *begin crawl
+       * 1 2 9.45 234 -01
+       * *end crawl
+       * *begin crawl    # <- warning here
+       * 2 3 7.67 223 -03
+       * *end crawl
+       * 
+       * If you're unsure what "deprecated" means, see:
+       * http://en.wikipedia.org/wiki/Deprecation */
+      compile_warning(/*Reentering an existing survey is deprecated*/29);
+      compile_warning_pfx(survey, /*Originally entered here*/30);
+      if (++reenter_depr_count == 5)
+	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
    } else {
-      tag->sflags |= BIT(SFLAGS_PREFIX_ENTERED);
-      tag->filename = file.filename;
-      tag->line = file.line;
+      survey->sflags |= BIT(SFLAGS_PREFIX_ENTERED);
+      survey->filename = file.filename;
+      survey->line = file.line;
    }
 }
 
@@ -527,18 +550,20 @@ static void
 cmd_prefix(void)
 {
    static int prefix_depr_count = 0;
-   prefix *tag;
+   prefix *survey;
    /* Issue warning first, so "*prefix \" warns first that *prefix is
     * deprecated and then that ROOT is...
     */
    if (prefix_depr_count < 5) {
+      /* TRANSLATORS: If you're unsure what "deprecated" means, see:
+       * http://en.wikipedia.org/wiki/Deprecation */
       compile_warning(-/**prefix is deprecated - use *begin and *end instead*/6);
       if (++prefix_depr_count == 5)
 	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
    }
-   tag = read_prefix(PFX_SURVEY|PFX_ALLOW_ROOT);
-   pcs->Prefix = tag;
-   check_reentry(tag);
+   survey = read_prefix(PFX_SURVEY|PFX_ALLOW_ROOT);
+   pcs->Prefix = survey;
+   check_reentry(survey);
 }
 #endif
 
@@ -567,7 +592,7 @@ bad:
 static void
 cmd_begin(void)
 {
-   prefix *tag;
+   prefix *survey;
    settings *pcsNew;
 
    pcsNew = osnew(settings);
@@ -576,11 +601,11 @@ cmd_begin(void)
    pcsNew->next = pcs;
    pcs = pcsNew;
 
-   tag = read_prefix(PFX_SURVEY|PFX_OPT|PFX_ALLOW_ROOT|PFX_WARN_SEPARATOR);
-   pcs->tag = tag;
-   if (tag) {
-      pcs->Prefix = tag;
-      check_reentry(tag);
+   survey = read_prefix(PFX_SURVEY|PFX_OPT|PFX_ALLOW_ROOT|PFX_WARN_SEPARATOR);
+   pcs->begin_survey = survey;
+   if (survey) {
+      pcs->Prefix = survey;
+      check_reentry(survey);
       f_export_ok = fTrue;
    }
 }
@@ -596,10 +621,14 @@ free_settings(settings *p) {
    if (!p->next || p->Translate != p->next->Translate)
       osfree(p->Translate - 1);
 
-   /* free meta is not used by parent, or in this block */
-   if (p->meta && p->meta != p->next->meta && p->meta->ref_count == 0)
+   /* free meta if not used by parent, or in this block */
+   if (p->meta && (!p->next || p->meta != p->next->meta) && p->meta->ref_count == 0)
        osfree(p->meta);
 
+   /* free proj if not used by parent, or as the output projection */
+   if (p->proj && (!p->next || p->proj != p->next->proj) && p->proj != proj_out)
+       pj_free(p->proj);
+
    osfree(p);
 }
 
@@ -607,7 +636,7 @@ static void
 cmd_end(void)
 {
    settings *pcsParent;
-   prefix *tag, *tagBegin;
+   prefix *survey, *begin_survey;
 
    pcsParent = pcs->next;
 
@@ -621,26 +650,37 @@ cmd_end(void)
       return;
    }
 
-   tagBegin = pcs->tag;
+   begin_survey = pcs->begin_survey;
 
    SVX_ASSERT(pcsParent);
    free_settings(pcs);
    pcs = pcsParent;
 
    /* note need to read using root *before* BEGIN */
-   tag = read_prefix(PFX_SURVEY|PFX_OPT|PFX_ALLOW_ROOT);
-   if (tag != tagBegin) {
-      if (tag) {
-	 if (!tagBegin) {
-	    /* "*begin" / "*end foo" */
-	    compile_error_skip(-/*Matching BEGIN tag has no prefix*/36);
+   survey = read_prefix(PFX_SURVEY|PFX_OPT|PFX_ALLOW_ROOT);
+   if (survey != begin_survey) {
+      if (survey) {
+	 if (!begin_survey) {
+	    /* TRANSLATORS: Used when a BEGIN command has no survey, but the
+	     * END command does, e.g.:
+	     *
+	     * *begin
+	     * 1 2 10.00 178 -01
+	     * *end entrance      <--[Message given here] */
+	    compile_error_skip(-/*Matching BEGIN command has no survey name*/36);
 	 } else {
-	    /* tag mismatch */
-	    compile_error_skip(-/*Prefix tag doesn’t match BEGIN*/193);
+	    /* TRANSLATORS: *BEGIN <survey> and *END <survey> should have the
+	     * same <survey> if it’s given at all */
+	    compile_error_skip(-/*Survey name doesn’t match BEGIN*/193);
 	 }
       } else {
-	 /* close tag omitted; open tag given */
-	 compile_warning(-/*Closing prefix omitted from END*/194);
+	 /* TRANSLATORS: Used when a BEGIN command has a survey name, but the
+	  * END command omits it, e.g.:
+	  *
+	  * *begin entrance
+	  * 1 2 10.00 178 -01
+	  * *end     <--[Message given here] */
+	 compile_warning(-/*Survey name omitted from END*/194);
       }
    }
 }
@@ -652,12 +692,18 @@ cmd_entrance(void)
    pfx->sflags |= BIT(SFLAGS_ENTRANCE);
 }
 
+static const prefix * first_fix_name = NULL;
+static const char * first_fix_filename;
+static unsigned first_fix_line;
+
 static void
 cmd_fix(void)
 {
    prefix *fix_name;
    node *stn = NULL;
-   static node *stnOmitAlready = NULL;
+   static prefix *name_omit_already = NULL;
+   static const char * name_omit_already_filename = NULL;
+   static unsigned int name_omit_already_line;
    real x, y, z;
    int nx, ny, nz;
    filepos fp;
@@ -681,18 +727,34 @@ cmd_fix(void)
       if (!isEol(ch) && !isComm(ch)) x = read_numeric(fFalse, &nx);
    }
    if (x == HUGE_REAL) {
-      if (stnOmitAlready) {
-	 if (fix_name != stnOmitAlready->name) {
-	    compile_error_skip(/*More than one FIX command with no coordinates*/56);
-	 } else {
-	    compile_warning(/*Same station fixed twice with no coordinates*/61);
-	 }
+      if (pcs->proj || proj_out) {
+	 compile_error_skip(/*Coordinates can't be omitted when coordinate system has been specified*/439);
 	 return;
       }
-      stn = StnFromPfx(fix_name);
+
+      if (fix_name == name_omit_already) {
+	 compile_warning(/*Same station fixed twice with no coordinates*/61);
+	 return;
+      }
+
+      /* TRANSLATORS: " *fix a " gives this message: */
       compile_warning(/*FIX command with no coordinates - fixing at (0,0,0)*/54);
+
+      if (name_omit_already) {
+	 /* TRANSLATORS: Emitted after second and subsequent "FIX command with
+	  * no coordinates - fixing at (0,0,0)" warnings.
+	  */
+	 compile_error_at(name_omit_already_filename,
+			  name_omit_already_line,
+			  /*Already had FIX command with no coordinates for station “%s”*/441,
+			  sprint_prefix(name_omit_already));
+      } else {
+	 name_omit_already = fix_name;
+	 name_omit_already_filename = file.filename;
+	 name_omit_already_line = file.line;
+      }
+
       x = y = z = (real)0.0;
-      stnOmitAlready = stn;
    } else {
       real sdx;
       y = read_numeric(fFalse, &ny);
@@ -755,9 +817,31 @@ cmd_fix(void)
 	 }
 	 return;
       }
-      stn = StnFromPfx(fix_name);
+
+      if (pcs->proj && proj_out) {
+	 int r = pj_transform(pcs->proj, proj_out, 1, 1, &x, &y, &z);
+	 if (r != 0) {
+	    compile_error(/*Failed to convert coordinates*/436);
+	    /* FIXME: report pj_strerrno(r) */
+	    printf("[%s]\n", pj_strerrno(r));
+	 }
+      } else if (pcs->proj) {
+	 compile_error(/*The input projection is set but the output projection isn't*/437);
+      } else if (proj_out) {
+	 compile_error(/*The output projection is set but the input projection isn't*/438);
+      }
    }
 
+   if (!first_fix_name) {
+      /* We track if we've fixed a station yet, and if so what the name of the
+       * first fix was, so that we can issue an error if the output coordinate
+       * system is set after fixing a station. */
+      first_fix_name = fix_name;
+      first_fix_filename = file.filename;
+      first_fix_line = file.line;
+   }
+
+   stn = StnFromPfx(fix_name);
    if (!fixed(stn)) {
       POS(stn, 0) = x;
       POS(stn, 1) = y;
@@ -770,6 +854,7 @@ cmd_fix(void)
       compile_error(/*Station already fixed or equated to a fixed point*/46);
       return;
    }
+   /* TRANSLATORS: *fix a 1 2 3 / *fix a 1 2 3 */
    compile_warning(/*Station already fixed at the same coordinates*/55);
 }
 
@@ -831,6 +916,8 @@ cmd_equate(void)
       name1 = read_prefix(PFX_STATION|PFX_ALLOW_ROOT|PFX_SUSPECT_TYPO|PFX_OPT);
       if (name1 == NULL) {
 	 if (fOnlyOneStn) {
+	    /* TRANSLATORS: EQUATE is a command name, so shouldn’t be
+	     * translated. */
 	    compile_error_skip(-/*Only one station in EQUATE command*/33);
 	 }
 	 return;
@@ -844,25 +931,32 @@ cmd_equate(void)
 static void
 report_missing_export(prefix *pfx, int depth)
 {
-   const char *filename_store = file.filename;
-   unsigned int line_store = file.line;
-   prefix *survey = pfx;
    char *s;
+   const char *p;
+   prefix *survey = pfx;
    int i;
    for (i = depth + 1; i; i--) {
       survey = survey->up;
       SVX_ASSERT(survey);
    }
    s = osstrdup(sprint_prefix(survey));
+   p = sprint_prefix(pfx);
    if (survey->filename) {
-      file.filename = survey->filename;
-      file.line = survey->line;
-   }
-   compile_error(/*Station “%s” not exported from survey “%s”*/26,
-		 sprint_prefix(pfx), s);
-   if (survey->filename) {
-      file.filename = filename_store;
-      file.line = line_store;
+      /* TRANSLATORS: A station must be exported out of each level it is in, so
+       * this would give "Station “\outer.inner.1” not exported from survey
+       * “\outer”)":
+       *
+       * *equate entrance outer.inner.1
+       * *begin outer
+       * *begin inner
+       * *export 1
+       * 1 2 1.23 045 -6
+       * *end inner
+       * *end outer */
+      compile_error_pfx(survey,
+			/*Station “%s” not exported from survey “%s”*/26, p, s);
+   } else {
+      compile_error(/*Station “%s” not exported from survey “%s”*/26, p, s);
    }
    osfree(s);
 }
@@ -902,6 +996,15 @@ cmd_export(void)
 	 if (pfx->min_export - 1 > depth) {
 	    report_missing_export(pfx, depth);
 	 } else if (pfx->min_export - 1 < depth) {
+	    /* TRANSLATORS: Here "station" is a survey station, not a train station.
+	     * 
+	     * Exporting a station twice gives this error:
+	     *
+	     * *begin example
+	     * *export 1
+	     * *export 1
+	     * 1 2 1.24 045 -6
+	     * *end example */
 	    compile_error(/*Station “%s” already exported*/66,
 			  sprint_prefix(pfx));
 	 }
@@ -1031,6 +1134,7 @@ cmd_data(void)
 
    if (style == STYLE_UNKNOWN) {
       file.lpos += strlen(buffer);
+      /* TRANSLATORS: e.g. trying to refer to an invalid FNORD data style */
       compile_error_skip(-/*Data style “%s” unknown*/65, buffer);
       return;
    }
@@ -1066,6 +1170,11 @@ cmd_data(void)
       /* Note: an unknown token is reported as trailing garbage */
       if (!TSTBIT(mask_all[style], d)) {
 	 file.lpos += strlen(buffer);
+	 /* TRANSLATORS: a data "style" is something like NORMAL, DIVING, etc.
+	  * a "reading" is one of FROM, TO, TAPE, COMPASS, CLINO for NORMAL
+	  * neither style nor reading is a keyword in the program This error
+	  * complains about a depth gauge reading in normal style, for example
+	  */
 	 compile_error_skip(-/*Reading “%s” not allowed in data style “%s”*/63,
 		       buffer, style_name);
 	 osfree(style_name);
@@ -1073,8 +1182,12 @@ cmd_data(void)
 	 return;
       }
       if (TSTBIT(mUsed, Newline) && TSTBIT(m_multi, d)) {
-	 /* e.g. "*data diving station newline tape depth compass" */
 	 file.lpos += strlen(buffer);
+	 /* TRANSLATORS: caused by e.g.
+	  *
+	  * *data diving station newline depth tape compass
+	  *
+	  * ("depth" needs to occur before "newline"). */
 	 compile_error_skip(-/*Reading “%s” must occur before NEWLINE*/225, buffer);
 	 osfree(style_name);
 	 osfree(new_order);
@@ -1086,6 +1199,8 @@ cmd_data(void)
       if (!((BIT(Ignore) | BIT(End) | BIT(IgnoreAll)) & BIT(d))) {
 	 if (TSTBIT(mUsed, d)) {
 	    file.lpos += strlen(buffer);
+	    /* TRANSLATORS: complains about a situation like trying to define
+	     * two from stations per leg */
 	    compile_error_skip(-/*Duplicate reading “%s”*/67, buffer);
 	    osfree(style_name);
 	    osfree(new_order);
@@ -1122,8 +1237,10 @@ cmd_data(void)
 	       break;
 	     case Newline:
 	       if (mUsed & ~m_multi) {
-		  /* e.g. "*data normal from to tape newline compass clino" */
 		  file.lpos += strlen(buffer);
+		  /* TRANSLATORS: e.g.
+		   *
+		   * *data normal from to tape newline compass clino */
 		  compile_error_skip(-/*NEWLINE can only be preceded by STATION, DEPTH, and COUNT*/226);
 		  osfree(style_name);
 		  osfree(new_order);
@@ -1131,6 +1248,9 @@ cmd_data(void)
 	       }
 	       if (k == 0) {
 		  file.lpos += strlen(buffer);
+		  /* TRANSLATORS: error from:
+		   *
+		   * *data normal newline from to tape compass clino */
 		  compile_error_skip(-/*NEWLINE can’t be the first reading*/222);
 		  osfree(style_name);
 		  osfree(new_order);
@@ -1166,6 +1286,9 @@ cmd_data(void)
 
    if (k >= 2 && new_order[k - 2] == Newline) {
       file.lpos += strlen(buffer);
+      /* TRANSLATORS: error from:
+       *
+       * *data normal from to tape compass clino newline */
       compile_error_skip(-/*NEWLINE can’t be the last reading*/223);
       osfree(style_name);
       osfree(new_order);
@@ -1184,10 +1307,11 @@ cmd_data(void)
    } else if (style == STYLE_PASSAGE) {
       /* Station doesn't mean "multiline" for STYLE_PASSAGE. */
    } else if (!TSTBIT(mUsed, Newline) && (m_multi & mUsed)) {
-      /* This is for when they write
+      /* TRANSLATORS: Error given by something like:
+       *
        * *data normal station tape compass clino
-       * (i.e. no newline, but interleaved readings)
-       */
+       *
+       * ("station" signifies interleaved data). */
       compile_error_skip(/*Interleaved readings, but no NEWLINE*/224);
       osfree(style_name);
       osfree(new_order);
@@ -1235,6 +1359,7 @@ cmd_data(void)
       /* Test should only fail with too few bits set, not too many */
       SVX_ASSERT((((mUsed &~ BIT(Newline)) | mask_optional[style])
 	      &~ mask[style]) == 0);
+      /* TRANSLATORS: i.e. not enough readings for the style. */
       compile_error_skip(/*Too few readings for data style “%s”*/64, style_name);
       osfree(style_name);
       osfree(new_order);
@@ -1278,6 +1403,10 @@ cmd_units(void)
 
    factor = read_numeric(fTrue, NULL);
    if (factor == 0.0) {
+      /* TRANSLATORS: error message given by "*units tape 0 feet" - it’s
+       * meaningless to say your tape is marked in "0 feet" (but you might
+       * measure distance by counting knots on a diving line, and tie them
+       * every "2 feet"). */
       compile_error_skip(-/**UNITS factor must be non-zero*/200);
       return;
    }
@@ -1322,10 +1451,14 @@ cmd_calibrate(void)
    /* check for declination scale */
    /* perhaps "*calibrate declination XXX" should be "*declination XXX" ? */
    if (TSTBIT(qmask, Q_DECLINATION) && sc != 1.0) {
+      /* TRANSLATORS: DECLINATION is a built-in keyword, so best not to
+       * translate */
       compile_error_skip(-/*Scale factor must be 1.0 for DECLINATION*/40);
       return;
    }
    if (sc == 0.0) {
+      /* TRANSLATORS: If the scale factor for an instrument is zero, then any
+       * reading would be mapped to zero, which doesn't make sense. */
       compile_error_skip(-/*Scale factor must be non-zero*/391);
       return;
    }
@@ -1350,6 +1483,8 @@ cmd_default(void)
    static int default_depr_count = 0;
 
    if (default_depr_count < 5) {
+      /* TRANSLATORS: If you're unsure what "deprecated" means, see:
+       * http://en.wikipedia.org/wiki/Deprecation */
       compile_warning(-/**DEFAULT is deprecated - use *CALIBRATE/DATA/SD/UNITS with argument DEFAULT instead*/20);
       if (++default_depr_count == 5)
 	 compile_warning(/*Further uses of this deprecated feature will not be reported*/95);
@@ -1476,6 +1611,251 @@ cmd_case(void)
    }
 }
 
+typedef enum {
+    CS_NONE = -1,
+    CS_CUSTOM,
+    CS_EPSG,
+    CS_ESRI,
+    CS_EUR,
+    CS_IJTSK,
+    CS_JTSK,
+    CS_LAT,
+    CS_LOCAL,
+    CS_LONG,
+    CS_OSGB,
+    CS_S_MERC,
+    CS_UTM
+} cs_class;
+
+static sztok cs_tab[] = {
+     {"CUSTOM", CS_CUSTOM},
+     {"EPSG",   CS_EPSG},	/* EPSG:<number> */
+     {"ESRI",   CS_ESRI},	/* ESRI:<number> */
+     {"EUR",    CS_EUR},	/* EUR79Z30 */
+     {"IJTSK",  CS_IJTSK},	/* IJTSK or IJTSK03 */
+     {"JTSK",   CS_JTSK},	/* JTSK or JTSK03 */
+     {"LAT",    CS_LAT},	/* LAT-LONG */
+     {"LOCAL",  CS_LOCAL},
+     {"LONG",   CS_LONG},	/* LONG-LAT */
+     {"OSGB",   CS_OSGB},	/* OSGB:<H, N, O, S or T><A-Z except I> */
+     {"S",      CS_S_MERC},	/* S-MERC */
+     {"UTM",    CS_UTM},	/* UTM<zone><N or S or nothing> */
+     {NULL,     CS_NONE}
+};
+
+static void
+cmd_cs(void)
+{
+   char * proj_str = NULL;
+   int proj_str_len;
+   cs_class cs;
+   int cs_sub = INT_MIN;
+   filepos fp;
+   bool output = fFalse;
+   enum { YES, NO, MAYBE } ok_for_output = YES;
+   static bool had_cs = fFalse;
+
+   if (!had_cs) {
+      had_cs = fTrue;
+      if (first_fix_name) {
+	 compile_error_at(first_fix_filename, first_fix_line,
+			  /*Station “%s” fixed before CS command first used*/442,
+			  sprint_prefix(first_fix_name));
+      }
+   }
+
+   get_pos(&fp);
+   /* Note get_token() only accepts letters - it'll stop at digits so "UTM12"
+    * will give token "UTM". */
+   get_token();
+   if (strcmp(ucbuffer, "OUT") == 0) {
+      output = fTrue;
+      get_token();
+   }
+   cs = match_tok(cs_tab, TABSIZE(cs_tab));
+   switch (cs) {
+      case CS_NONE:
+	 break;
+      case CS_CUSTOM:
+	 ok_for_output = MAYBE;
+	 read_string(&proj_str, &proj_str_len);
+	 cs_sub = 0;
+	 break;
+      case CS_EPSG: case CS_ESRI:
+	 ok_for_output = MAYBE;
+	 if (ch == ':' && isdigit(nextch())) {
+	    unsigned n = read_uint();
+	    if (n < 32768) {
+	       cs_sub = (int)n;
+	       /* FIXME: Implement */
+	    }
+	 }
+	 break;
+      case CS_EUR:
+	 if (isdigit(ch) &&
+	     read_uint() == 79 &&
+	     (ch == 'Z' || ch == 'z') &&
+	     isdigit(nextch()) &&
+	     read_uint() == 30) {
+	    cs_sub = 7930;
+	 }
+	 break;
+      case CS_JTSK:
+         ok_for_output = NO;
+	 /* FALLTHRU */
+      case CS_IJTSK:
+	 if (ch == '0') {
+	    if (nextch() == '3') {
+	       nextch();
+	       cs_sub = 3;
+	    }
+	 } else {
+	    cs_sub = 0;
+	 }
+	 break;
+      case CS_LAT: case CS_LONG:
+	 ok_for_output = NO;
+	 if (ch == '-') {
+	    nextch();
+	    get_token_no_blanks();
+	    cs_class cs2 = match_tok(cs_tab, TABSIZE(cs_tab));
+	    if ((cs ^ CS_LAT) == (cs2 ^ CS_LONG)) {
+		cs_sub = 0;
+	    }
+	 }
+	 break;
+      case CS_LOCAL:
+	 cs_sub = 0;
+	 break;
+      case CS_OSGB:
+	 if (ch == ':') {
+	    int uch1 = toupper(nextch());
+	    if (strchr("HNOST", uch1)) {
+	       int uch2 = toupper(nextch());
+	       if (uch2 >= 'A' && uch2 <= 'Z' && uch2 != 'I') {
+		  nextch();
+		  if (uch2 > 'I') --uch2;
+		  cs_sub = (uch1 - 'A') * 25 + (uch2 - 'A');
+	       }
+	    }
+	 }
+	 break;
+      case CS_S_MERC:
+         /* FIXME: Is this ok_for_output? */
+	 if (ch == '-') {
+	    get_token_no_blanks();
+	    if (strcmp(ucbuffer, "MERC") == 0) {
+	       cs_sub = 0;
+	    }
+	 }
+	 break;
+      case CS_UTM:
+	 if (isdigit(ch)) {
+	    unsigned n = read_uint();
+	    if (n >= 1 && n <= 60) {
+	       int uch = toupper(ch);
+	       cs_sub = (int)n;
+	       if (uch == 'S') {
+		  nextch();
+		  cs_sub = -cs_sub;
+	       } else if (uch == 'N') {
+		  nextch();
+	       }
+	    }
+	 }
+	 break;
+   }
+   if (cs_sub == INT_MIN || isalnum(ch)) {
+      set_pos(&fp);
+      compile_error_skip(-/*Unknown coordinate system*/434);
+      return;
+   }
+   /* Actually handle the cs */
+   switch (cs) {
+      case CS_CUSTOM:
+	 break;
+      case CS_EPSG:
+      case CS_ESRI:
+      case CS_EUR:
+      case CS_IJTSK:
+      case CS_JTSK:
+      case CS_LAT:
+      case CS_LOCAL:
+      case CS_LONG:
+      case CS_OSGB:
+      case CS_S_MERC:
+      default:
+	 /* FIXME: Handle these too... */
+	 /* printf("CS %d:%d\n", (int)cs, cs_sub); */
+	 set_pos(&fp);
+	 compile_error_skip(-/*Unknown coordinate system*/434);
+	 return;
+      case CS_UTM:
+	 proj_str = osmalloc(64);
+	 if (cs_sub > 0) {
+	    sprintf(proj_str, "+proj=utm +ellps=WGS84 +datum=WGS84 +units=m +zone=%d", cs_sub);
+	 } else {
+	    sprintf(proj_str, "+proj=utm +ellps=WGS84 +datum=WGS84 +units=m +zone=%d +south", -cs_sub);
+	 }
+	 break;
+   }
+
+   if (output) {
+      if (ok_for_output == NO) {
+	 set_pos(&fp);
+	 compile_error_skip(-/*Coordinate system unsuitable for output*/435);
+	 return;
+      }
+
+      /* If the output projection is already set, we still need to create the
+       * projection object for a custom projection, so we can report errors.
+       * But if the string is identical, we know it's valid.
+       */
+      if (!proj_out ||
+	  (ok_for_output == MAYBE && strcmp(proj_str, proj_str_out) != 0)) {
+	 projPJ pj = pj_init_plus(proj_str);
+	 if (!pj) {
+	    set_pos(&fp);
+	    compile_error_skip(-/*Invalid coordinate system: %s*/443,
+			       pj_strerrno(pj_errno));
+	    return;
+	 }
+	 if (ok_for_output == MAYBE && pj_is_latlong(pj)) {
+	    compile_error_skip(-/*Coordinate system unsuitable for output*/435);
+	    return;
+	 }
+	 if (proj_out) {
+	    pj_free(pj);
+	    osfree(proj_str);
+	 } else {
+	    proj_out = pj;
+	    proj_str_out = proj_str;
+	 }
+      }
+   } else {
+      projPJ pj;
+      if (proj_str_out && strcmp(proj_str, proj_str_out) == 0) {
+	 /* Same as the current output projection. */
+	 pj = proj_out;
+      } else {
+	 pj = pj_init_plus(proj_str);
+	 if (!pj) {
+	    set_pos(&fp);
+	    compile_error_skip(-/*Invalid coordinate system: %s*/443,
+			       pj_strerrno(pj_errno));
+	    return;
+	 }
+      }
+
+      /* Free proj if not used by parent, or as the output projection. */
+      settings * p = pcs;
+      if (p->proj && (!p->next || p->proj != p->next->proj))
+	 if (p->proj != proj_out)
+	    pj_free(p->proj);
+      p->proj = pj;
+   }
+}
+
 static sztok infer_tab[] = {
      { "EQUATES",	INFER_EQUATES },
      { "EXPORTS",	INFER_EXPORTS },
@@ -1565,6 +1945,11 @@ cmd_require(void)
 	    nextch();
 	 }
 	 v[i] = '\0';
+	 /* TRANSLATORS: Feel free to translate as "or newer" instead of "or
+	  * greater" if that gives a more natural translation.  It's
+	  * technically not quite right when there are parallel active release
+	  * series (e.g. Survex 1.0.40 was released *after* 1.2.0), but this
+	  * seems unlikely to confuse users. */
 	 fatalerror_in_file(file.filename, file.line, /*Survex version %s or greater required to process this survey data.*/2, v);
       }
       if (ch != '.') break;
@@ -1643,6 +2028,7 @@ static cmd_fn cmd_funcs[] = {
    cmd_calibrate,
    cmd_case,
    skipline, /*cmd_copyright,*/
+   cmd_cs,
    cmd_data,
    cmd_date,
 #ifndef NO_DEPRECATED
@@ -1686,6 +2072,13 @@ handle_command(void)
    switch (cmdtok) {
     case CMD_EXPORT:
       if (!f_export_ok)
+	 /* TRANSLATORS: The *EXPORT command is only valid just after *BEGIN
+	  * <SURVEY>, so this would generate this error:
+	  *
+	  * *begin fred
+	  * 1 2 1.23 045 -6
+	  * *export 2
+	  * *end fred */
 	 compile_error(/**EXPORT must immediately follow “*BEGIN <SURVEY>”*/57);
       break;
     case CMD_COPYRIGHT:
