@@ -85,9 +85,14 @@ static const int SCALE_BAR_HEIGHT = 12;
 static const gla_colour TEXT_COLOUR = col_GREEN;
 static const gla_colour HERE_COLOUR = col_WHITE;
 static const gla_colour NAME_COLOUR = col_GREEN;
+static const gla_colour SEL_COLOUR = col_WHITE;
 
+// Number of entries across and down the hit-test grid:
 #define HITTEST_SIZE 20
 
+// How close the pointer needs to be to a station to be considered:
+#define MEASURE_THRESHOLD 7
+
 // vector for lighting angle
 static const Vector3 light(.577, .577, .577);
 
@@ -143,13 +148,16 @@ GfxCore::GfxCore(MainFrm* parent, wxWindow* parent_win, GUIControl* control) :
     m_Degrees(false),
     m_Metric(false),
     m_Percent(false),
+    m_HitTestDebug(false),
+    m_PointGrid(NULL),
     m_HitTestGridValid(false),
     m_here_is_temporary(false),
     presentation_mode(0),
     pres_reverse(false),
     pres_speed(0.0),
     movie(NULL),
-    current_cursor(GfxCore::CURSOR_DEFAULT)
+    current_cursor(GfxCore::CURSOR_DEFAULT),
+    sqrd_measure_threshold(sqrd(MEASURE_THRESHOLD))
 {
     AddQuad = &GfxCore::AddQuadrilateralDepth;
     AddPoly = &GfxCore::AddPolylineDepth;
@@ -159,9 +167,6 @@ GfxCore::GfxCore(MainFrm* parent, wxWindow* parent_win, GUIControl* control) :
     m_here.Invalidate();
     m_there.Invalidate();
 
-    // Initialise grid for hit testing.
-    m_PointGrid = new list<LabelInfo*>[HITTEST_SIZE * HITTEST_SIZE];
-
     for (int pen = 0; pen < NUM_COLOUR_BANDS + 1; ++pen) {
 	m_Pens[pen].SetColour(REDS[pen] / 255.0,
 			      GREENS[pen] / 255.0,
@@ -382,6 +387,39 @@ void GfxCore::OnPaint(wxPaintEvent&)
 	    }
 	}
 
+	if (m_HitTestDebug) {
+	    // Show the hit test grid bucket sizes...
+	    SetColour(m_HitTestGridValid ? col_LIGHT_GREY : col_DARK_GREY);
+	    if (m_PointGrid) {
+		for (int i = 0; i != HITTEST_SIZE; ++i) {
+		    int x = (GetXSize() + 1) * i / HITTEST_SIZE + 2;
+		    for (int j = 0; j != HITTEST_SIZE; ++j) {
+			int square = i + j * HITTEST_SIZE;
+			unsigned long bucket_size = m_PointGrid[square].size();
+			if (bucket_size) {
+			    int y = (GetYSize() + 1) * (HITTEST_SIZE - 1 - j) / HITTEST_SIZE;
+			    DrawIndicatorText(x, y, wxString::Format("%lu", bucket_size));
+			}
+		    }
+		}
+	    }
+
+	    EnableDashedLines();
+	    BeginLines();
+	    for (int i = 0; i != HITTEST_SIZE; ++i) {
+		int x = (GetXSize() + 1) * i / HITTEST_SIZE;
+		PlaceIndicatorVertex(x, 0);
+		PlaceIndicatorVertex(x, GetYSize());
+	    }
+	    for (int j = 0; j != HITTEST_SIZE; ++j) {
+		int y = (GetYSize() + 1) * (HITTEST_SIZE - 1 - j) / HITTEST_SIZE;
+		PlaceIndicatorVertex(0, y);
+		PlaceIndicatorVertex(GetXSize(), y);
+	    }
+	    EndLines();
+	    DisableDashedLines();
+	}
+
 	// Draw indicators.
 	//
 	// There's no advantage in generating an OpenGL list for the
@@ -392,7 +430,19 @@ void GfxCore::OnPaint(wxPaintEvent&)
 	// right...
 	DrawIndicators();
 
-	if (MeasuringLineActive()) {
+	if (zoombox.active()) {
+	    SetColour(SEL_COLOUR);
+	    EnableDashedLines();
+	    BeginPolyline();
+	    glaCoord Y = GetYSize();
+	    PlaceIndicatorVertex(zoombox.x1, Y - zoombox.y1);
+	    PlaceIndicatorVertex(zoombox.x1, Y - zoombox.y2);
+	    PlaceIndicatorVertex(zoombox.x2, Y - zoombox.y2);
+	    PlaceIndicatorVertex(zoombox.x2, Y - zoombox.y1);
+	    PlaceIndicatorVertex(zoombox.x1, Y - zoombox.y1);
+	    EndPolyline();
+	    DisableDashedLines();
+	} else if (MeasuringLineActive()) {
 	    // Draw "here" and "there".
 	    double hx, hy;
 	    SetColour(HERE_COLOUR);
@@ -642,9 +692,13 @@ void GfxCore::Draw2dIndicators()
 	int brg_unit;
 	if (m_Degrees) {
 	    value = int(m_PanAngle);
+	    /* TRANSLATORS: degree symbol - probably should be translated to
+	     * itself. */
 	    brg_unit = /*°*/344;
 	} else {
 	    value = int(m_PanAngle * 200.0 / 180.0);
+	    /* TRANSLATORS: symbol for grad (400 grad = 360 degrees = full
+	     * circle). */
 	    brg_unit = /*ᵍ*/76;
 	}
 	str.Printf(wxT("%03d"), value);
@@ -658,79 +712,105 @@ void GfxCore::Draw2dIndicators()
     }
 
     if (m_Clino) {
-	int angle;
-	wxString str;
-	int width;
-	int unit;
-	if (m_Percent) {
-	    static int zero_width = 0;
-	    if (!zero_width) {
-		GetTextExtent(wxT("0"), &zero_width, NULL);
+	if (m_TiltAngle == -90.0) {
+	    // TRANSLATORS: Label used for "clino" in Aven when the view is
+	    // from directly above.
+	    wxString str = wmsg(/*Plan*/432);
+	    static int width = 0;
+	    if (!width) {
+		GetTextExtent(str, &width, NULL);
 	    }
-	    width = zero_width;
-	    if (m_TiltAngle > 89.99) {
-		angle = 1000000;
-	    } else if (m_TiltAngle < -89.99) {
-		angle = -1000000;
-	    } else {
-		angle = int(100 * tan(rad(m_TiltAngle)));
+	    int x = elev_centre_x - width / 2;
+	    DrawIndicatorText(x, y_off + height / 2, str);
+	} else if (m_TiltAngle == 90.0) {
+	    // TRANSLATORS: Label used for "clino" in Aven when the view is
+	    // from directly below.
+	    wxString str = wmsg(/*Kiwi Plan*/433);
+	    static int width = 0;
+	    if (!width) {
+		GetTextExtent(str, &width, NULL);
 	    }
-	    if (angle > 99999 || angle < -99999) {
-		str = angle > 0 ? wxT("+") : wxT("-");
-		str += wmsg(/*∞*/431);
-	    } else {
-		str = angle ? wxString::Format(wxT("%+03d"), angle) : wxT("0");
-	    }
-	    unit = /*%*/96;
-	} else if (m_Degrees) {
-	    static int zero_zero_width = 0;
-	    if (!zero_zero_width) {
-		GetTextExtent(wxT("00"), &zero_zero_width, NULL);
-	    }
-	    width = zero_zero_width;
-	    angle = int(m_TiltAngle);
-	    str = angle ? wxString::Format(wxT("%+03d"), angle) : wxT("00");
-	    unit = /*°*/344;
+	    int x = elev_centre_x - width / 2;
+	    DrawIndicatorText(x, y_off + height / 2, str);
 	} else {
-	    width = triple_zero_width;
-	    angle = int(m_TiltAngle * 200.0 / 180.0);
-	    str = angle ? wxString::Format(wxT("%+04d"), angle) : wxT("000");
-	    unit = /*ᵍ*/76;
-	}
-
-	int sign_offset = 0;
-	if (unit == /*%*/96) {
-	    // Right align % since the width changes so much.
-	    GetTextExtent(str, &sign_offset, NULL);
-	    sign_offset -= width;
-	} else if (angle < 0) {
-	    // Adjust horizontal position so the left of the first digit is
-	    // always in the same place.
-	    static int minus_width = 0;
-	    if (!minus_width) {
-		GetTextExtent(wxT("-"), &minus_width, NULL);
+	    int angle;
+	    wxString str;
+	    int width;
+	    int unit;
+	    if (m_Percent) {
+		static int zero_width = 0;
+		if (!zero_width) {
+		    GetTextExtent(wxT("0"), &zero_width, NULL);
+		}
+		width = zero_width;
+		if (m_TiltAngle > 89.99) {
+		    angle = 1000000;
+		} else if (m_TiltAngle < -89.99) {
+		    angle = -1000000;
+		} else {
+		    angle = int(100 * tan(rad(m_TiltAngle)));
+		}
+		if (angle > 99999 || angle < -99999) {
+		    str = angle > 0 ? wxT("+") : wxT("-");
+		    /* TRANSLATORS: used for the percentage gradient on
+		     * vertical angles. */
+		    str += wmsg(/*∞*/431);
+		} else {
+		    str = angle ? wxString::Format(wxT("%+03d"), angle) : wxT("0");
+		}
+		/* TRANSLATORS: symbol for percentage gradient (100% = 45
+		 * degrees = 50 grad). */
+		unit = /*%*/96;
+	    } else if (m_Degrees) {
+		static int zero_zero_width = 0;
+		if (!zero_zero_width) {
+		    GetTextExtent(wxT("00"), &zero_zero_width, NULL);
+		}
+		width = zero_zero_width;
+		angle = int(m_TiltAngle);
+		str = angle ? wxString::Format(wxT("%+03d"), angle) : wxT("00");
+		unit = /*°*/344;
+	    } else {
+		width = triple_zero_width;
+		angle = int(m_TiltAngle * 200.0 / 180.0);
+		str = angle ? wxString::Format(wxT("%+04d"), angle) : wxT("000");
+		unit = /*ᵍ*/76;
 	    }
-	    sign_offset = minus_width;
-	} else if (angle > 0) {
-	    // Adjust horizontal position so the left of the first digit is
-	    // always in the same place.
-	    static int plus_width = 0;
-	    if (!plus_width) {
-		GetTextExtent(wxT("+"), &plus_width, NULL);
+
+	    int sign_offset = 0;
+	    if (unit == /*%*/96) {
+		// Right align % since the width changes so much.
+		GetTextExtent(str, &sign_offset, NULL);
+		sign_offset -= width;
+	    } else if (angle < 0) {
+		// Adjust horizontal position so the left of the first digit is
+		// always in the same place.
+		static int minus_width = 0;
+		if (!minus_width) {
+		    GetTextExtent(wxT("-"), &minus_width, NULL);
+		}
+		sign_offset = minus_width;
+	    } else if (angle > 0) {
+		// Adjust horizontal position so the left of the first digit is
+		// always in the same place.
+		static int plus_width = 0;
+		if (!plus_width) {
+		    GetTextExtent(wxT("+"), &plus_width, NULL);
+		}
+		sign_offset = plus_width;
 	    }
-	    sign_offset = plus_width;
-	}
 
-	str += wmsg(unit);
-	DrawIndicatorText(elev_centre_x - sign_offset - width / 2, y_off, str);
+	    str += wmsg(unit);
+	    DrawIndicatorText(elev_centre_x - sign_offset - width / 2, y_off, str);
 
-	str = wmsg(/*Elevation*/118);
-	static int elevation_width = 0;
-	if (!elevation_width) {
-	    GetTextExtent(str, &elevation_width, NULL);
+	    str = wmsg(/*Elevation*/118);
+	    static int elevation_width = 0;
+	    if (!elevation_width) {
+		GetTextExtent(str, &elevation_width, NULL);
+	    }
+	    int x = elev_centre_x - elevation_width / 2;
+	    DrawIndicatorText(x, y_off + height, str);
 	}
-	int x = elev_centre_x - elevation_width / 2;
-	DrawIndicatorText(x, y_off + height, str);
     }
 }
 
@@ -966,6 +1046,8 @@ void GfxCore::DrawDateKey()
 
     wxString other;
     if (!m_Parent->HasCompleteDateInfo()) {
+	/* TRANSLATORS: Used in the "colour key" for "colour by date" if there
+	 * are surveys without date information. */
 	other = wmsg(/*Undated*/221);
     }
 
@@ -988,6 +1070,8 @@ void GfxCore::DrawErrorKey()
     }
 
     // Always show the "Not in loop" legend for now (FIXME).
+    /* TRANSLATORS: Used in the "colour key" for "colour by error" for surveys
+     * which aren’t part of a loop and so have no error information. */
     DrawColourKey(num_bands, wmsg(/*Not in loop*/290), wxString());
 }
 
@@ -1056,11 +1140,26 @@ void GfxCore::DrawScaleBar()
 	Double km = size_snap * 1e-3;
 	if (km >= 1.0) {
 	    size_snap = km;
+	    /* TRANSLATORS: abbreviation for "kilometres" (unit of length),
+	     * used e.g.  "5km".
+	     *
+	     * If there should be a space between the number and this, include
+	     * one in the translation. */
 	    units = /*km*/423;
 	} else if (size_snap >= 1.0) {
+	    /* TRANSLATORS: abbreviation for "metres" (unit of length), used
+	     * e.g. "10m".
+	     * 
+	     * If there should be a space between the number and this, include
+	     * one in the translation. */
 	    units = /*m*/424;
 	} else {
 	    size_snap *= 1e2;
+	    /* TRANSLATORS: abbreviation for "centimetres" (unit of length),
+	     * used e.g.  "50cm".
+	     * 
+	     * If there should be a space between the number and this, include
+	     * one in the translation. */
 	    units = /*cm*/425;
 	}
     } else {
@@ -1069,14 +1168,34 @@ void GfxCore::DrawScaleBar()
 	if (miles >= 1.0) {
 	    size_snap = miles;
 	    if (size_snap >= 2.0) {
+		/* TRANSLATORS: abbreviation for "miles" (unit of length,
+		 * plural), used e.g.  "2 miles".
+		 * 
+		 * If there should be a space between the number and this,
+		 * include one in the translation. */
 		units = /* miles*/426;
 	    } else {
+		/* TRANSLATORS: abbreviation for "mile" (unit of length,
+		 * singular), used e.g.  "1 mile".
+		 * 
+		 * If there should be a space between the number and this,
+		 * include one in the translation. */
 		units = /* mile*/427;
 	    }
 	} else if (size_snap >= 1.0) {
+	    /* TRANSLATORS: abbreviation for "feet" (unit of length), used e.g.
+	     * as "10ft".
+	     * 
+	     * If there should be a space between the number and this, include
+	     * one in the translation. */
 	    units = /*ft*/428;
 	} else {
 	    size_snap *= 12.0;
+	    /* TRANSLATORS: abbreviation for "inches" (unit of length), used
+	     * e.g. as "6in".
+	     * 
+	     * If there should be a space between the number and this, include
+	     * one in the translation. */
 	    units = /*in*/429;
 	}
     }
@@ -1108,11 +1227,11 @@ bool GfxCore::CheckHitTestGrid(const wxPoint& point, bool centre)
 
     if (!m_HitTestGridValid) CreateHitTestGrid();
 
-    int grid_x = (point.x * (HITTEST_SIZE - 1)) / GetXSize();
-    int grid_y = (point.y * (HITTEST_SIZE - 1)) / GetYSize();
+    int grid_x = point.x * HITTEST_SIZE / (GetXSize() + 1);
+    int grid_y = point.y * HITTEST_SIZE / (GetYSize() + 1);
 
     LabelInfo *best = NULL;
-    int dist_sqrd = 25;
+    int dist_sqrd = sqrd_measure_threshold;
     int square = grid_x + grid_y * HITTEST_SIZE;
     list<LabelInfo*>::iterator iter = m_PointGrid[square].begin();
 
@@ -1496,9 +1615,14 @@ void GfxCore::SetThere(const Point &p)
 
 void GfxCore::CreateHitTestGrid()
 {
-    // Clear hit-test grid.
-    for (int i = 0; i < HITTEST_SIZE * HITTEST_SIZE; i++) {
-	m_PointGrid[i].clear();
+    if (!m_PointGrid) {
+	// Initialise hit-test grid.
+	m_PointGrid = new list<LabelInfo*>[HITTEST_SIZE * HITTEST_SIZE];
+    } else {
+	// Clear hit-test grid.
+	for (int i = 0; i < HITTEST_SIZE * HITTEST_SIZE; i++) {
+	    m_PointGrid[i].clear();
+	}
     }
 
     // Fill the grid.
@@ -1524,8 +1648,8 @@ void GfxCore::CreateHitTestGrid()
 	cy = GetYSize() - cy;
 
 	// On-screen, so add to hit-test grid...
-	int grid_x = int((cx * (HITTEST_SIZE - 1)) / GetXSize());
-	int grid_y = int((cy * (HITTEST_SIZE - 1)) / GetYSize());
+	int grid_x = int(cx * HITTEST_SIZE / (GetXSize() + 1));
+	int grid_y = int(cy * HITTEST_SIZE / (GetYSize() + 1));
 
 	m_PointGrid[grid_x + grid_y * HITTEST_SIZE].push_back(label);
     }
@@ -1943,9 +2067,10 @@ bool GfxCore::ShowingElevation() const
 
 bool GfxCore::ShowingMeasuringLine() const
 {
-    // Determine if the measuring line is being shown.
+    // Determine if the measuring line is being shown.  Only check if "there"
+    // is valid, since that means the measuring line anchor is out.
 
-    return (m_there.IsValid() && m_here.IsValid());
+    return m_there.IsValid();
 }
 
 void GfxCore::ToggleFlag(bool* flag, int update)
@@ -1956,6 +2081,7 @@ void GfxCore::ToggleFlag(bool* flag, int update)
     } else if (update == UPDATE_BLOBS_AND_CROSSES) {
 	UpdateBlobs();
 	InvalidateList(LIST_CROSSES);
+	m_HitTestGridValid = false;
     }
     ForceRefresh();
 }
@@ -1975,6 +2101,17 @@ int GfxCore::GetNumExportedPts() const
     return m_Parent->GetNumExportedPts();
 }
 
+void GfxCore::ToggleFatFinger()
+{
+    if (sqrd_measure_threshold == sqrd(MEASURE_THRESHOLD)) {
+	sqrd_measure_threshold = sqrd(5 * MEASURE_THRESHOLD);
+	wxMessageBox("Fat finger enabled", "Aven Debug", wxOK | wxICON_INFORMATION);
+    } else {
+	sqrd_measure_threshold = sqrd(MEASURE_THRESHOLD);
+	wxMessageBox("Fat finger disabled", "Aven Debug", wxOK | wxICON_INFORMATION);
+    }
+}
+
 void GfxCore::ClearTreeSelection()
 {
     m_Parent->ClearTreeSelection();
@@ -2214,7 +2351,7 @@ void GfxCore::PlaceVertexWithColour(const Vector3 & v,
 				    glaTexCoord tex_x, glaTexCoord tex_y,
 				    Double factor)
 {
-    SetColour(GetSurfacePen(), factor); // FIXME : assumes surface pen is white!
+    SetColour(col_WHITE, factor);
     PlaceVertex(v, tex_x, tex_y);
 }
 
@@ -2327,7 +2464,7 @@ Double GfxCore::GetDepthBoundaryBetweenBands(int a, int b) const
 void GfxCore::AddPolyline(const traverse & centreline)
 {
     BeginPolyline();
-    SetColour(GetSurfacePen());
+    SetColour(col_WHITE);
     vector<PointInfo>::const_iterator i = centreline.begin();
     PlaceVertex(*i);
     ++i;
@@ -2437,7 +2574,7 @@ void GfxCore::SetColourFromDate(int date, Double factor)
 
     if (date == -1) {
 	// Undated.
-	SetColour(GetSurfacePen(), factor);
+	SetColour(col_WHITE, factor);
 	return;
     }
 
@@ -2527,7 +2664,7 @@ void GfxCore::SetColourFromError(double E, Double factor)
     // Set the drawing colour based on an error value.
 
     if (E < 0) {
-	SetColour(GetSurfacePen(), factor);
+	SetColour(col_WHITE, factor);
 	return;
     }
 
@@ -2934,21 +3071,24 @@ bool GfxCore::ExportMovie(const wxString & fnm)
 
 void
 GfxCore::OnPrint(const wxString &filename, const wxString &title,
-		 const wxString &datestamp, time_t datestamp_numeric)
+		 const wxString &datestamp, time_t datestamp_numeric,
+		 const wxString &cs_proj,
+		 bool close_after_print)
 {
     svxPrintDlg * p;
-    p = new svxPrintDlg(m_Parent, filename, title,
+    p = new svxPrintDlg(m_Parent, filename, title, cs_proj,
 			datestamp, datestamp_numeric,
 			m_PanAngle, m_TiltAngle,
 			m_Names, m_Crosses, m_Legs, m_Surface, m_Tubes,
 			m_Entrances, m_FixedPts, m_ExportedPts,
-			true);
+			true, close_after_print);
     p->Show(true);
 }
 
 void
 GfxCore::OnExport(const wxString &filename, const wxString &title,
-		  const wxString &datestamp, time_t datestamp_numeric)
+		  const wxString &datestamp, time_t datestamp_numeric,
+		  const wxString &cs_proj)
 {
     // Fill in "right_bearing" for each cross-section.
     list<vector<XSect> >::iterator trav = m_Parent->tubes_begin();
@@ -2959,7 +3099,7 @@ GfxCore::OnExport(const wxString &filename, const wxString &title,
     }
 
     svxPrintDlg * p;
-    p = new svxPrintDlg(m_Parent, filename, title,
+    p = new svxPrintDlg(m_Parent, filename, title, cs_proj,
 			datestamp, datestamp_numeric,
 			m_PanAngle, m_TiltAngle,
 			m_Names, m_Crosses, m_Legs, m_Surface, m_Tubes,
@@ -3060,12 +3200,18 @@ bool GfxCore::HandleRClick(wxPoint point)
     if (PointWithinCompass(point)) {
 	// Pop up menu.
 	wxMenu menu;
+	/* TRANSLATORS: View *looking* North */
 	menu.Append(menu_ORIENT_MOVE_NORTH, wmsg(/*View &North*/240));
+	/* TRANSLATORS: View *looking* East */
 	menu.Append(menu_ORIENT_MOVE_EAST, wmsg(/*View &East*/241));
+	/* TRANSLATORS: View *looking* South */
 	menu.Append(menu_ORIENT_MOVE_SOUTH, wmsg(/*View &South*/242));
+	/* TRANSLATORS: View *looking* West */
 	menu.Append(menu_ORIENT_MOVE_WEST, wmsg(/*View &West*/243));
 	menu.AppendSeparator();
+	/* TRANSLATORS: Menu item which turns off the "north arrow" in aven. */
 	menu.AppendCheckItem(menu_IND_COMPASS, wmsg(/*&Hide Compass*/387));
+	/* TRANSLATORS: tickable menu item in View menu */
 	menu.AppendCheckItem(menu_CTL_DEGREES, wmsg(/*&Degrees*/343));
 	menu.Connect(wxEVT_COMMAND_MENU_SELECTED, (wxObjectEventFunction)&wxEvtHandler::ProcessEvent, NULL, m_Parent->GetEventHandler());
 	PopupMenu(&menu);
@@ -3078,8 +3224,17 @@ bool GfxCore::HandleRClick(wxPoint point)
 	menu.Append(menu_ORIENT_PLAN, wmsg(/*&Plan View*/248));
 	menu.Append(menu_ORIENT_ELEVATION, wmsg(/*Ele&vation*/249));
 	menu.AppendSeparator();
+	/* TRANSLATORS: Menu item which turns off the tilt indicator in aven. */
 	menu.AppendCheckItem(menu_IND_CLINO, wmsg(/*&Hide Clino*/384));
+	/* TRANSLATORS: tickable menu item in View menu.
+	 *
+	 * Degrees are the angular measurement where there are 360 in a full
+	 * circle. */
 	menu.AppendCheckItem(menu_CTL_DEGREES, wmsg(/*&Degrees*/343));
+	/* TRANSLATORS: tickable menu item in View menu.
+	 *
+	 * Show the tilt of the survey as a percentage gradient (100% = 45
+	 * degrees = 50 grad). */
 	menu.AppendCheckItem(menu_CTL_PERCENT, wmsg(/*&Percent*/430));
 	menu.Connect(wxEVT_COMMAND_MENU_SELECTED, (wxObjectEventFunction)&wxEvtHandler::ProcessEvent, NULL, m_Parent->GetEventHandler());
 	PopupMenu(&menu);
@@ -3089,7 +3244,9 @@ bool GfxCore::HandleRClick(wxPoint point)
     if (PointWithinScaleBar(point)) {
 	// Pop up menu.
 	wxMenu menu;
+	/* TRANSLATORS: Menu item which turns off the scale bar in aven. */
 	menu.AppendCheckItem(menu_IND_SCALE_BAR, wmsg(/*&Hide scale bar*/385));
+	/* TRANSLATORS: tickable menu item in View menu */
 	menu.AppendCheckItem(menu_CTL_METRIC, wmsg(/*&Metric*/342));
 	menu.Connect(wxEVT_COMMAND_MENU_SELECTED, (wxObjectEventFunction)&wxEvtHandler::ProcessEvent, NULL, m_Parent->GetEventHandler());
 	PopupMenu(&menu);
@@ -3103,6 +3260,9 @@ bool GfxCore::HandleRClick(wxPoint point)
 	menu.AppendCheckItem(menu_VIEW_COLOUR_BY_DATE, wmsg(/*Colour by D&ate*/293));
 	menu.AppendCheckItem(menu_VIEW_COLOUR_BY_ERROR, wmsg(/*Colour by E&rror*/289));
 	menu.AppendSeparator();
+	/* TRANSLATORS: Menu item which turns off the colour key.
+	 * The "Colour Key" is the thing in aven showing which colour
+	 * corresponds to which depth, date, survey closure error, etc. */
 	menu.AppendCheckItem(menu_IND_COLOUR_KEY, wmsg(/*&Hide colour key*/386));
 	if (m_ColourBy == COLOUR_BY_DEPTH)
 	    menu.AppendCheckItem(menu_CTL_METRIC, wmsg(/*&Metric*/342));
@@ -3113,3 +3273,49 @@ bool GfxCore::HandleRClick(wxPoint point)
 
     return false;
 }
+
+void GfxCore::SetZoomBox(wxPoint p1, wxPoint p2, bool centred, bool aspect)
+{
+    if (centred) {
+	p1.x = p2.x + (p1.x - p2.x) * 2;
+	p1.y = p2.y + (p1.y - p2.y) * 2;
+    }
+    if (aspect) {
+#if 0 // FIXME: This needs more work.
+	int sx = GetXSize();
+	int sy = GetYSize();
+	int dx = p1.x - p2.x;
+	int dy = p1.y - p2.y;
+	int dy_new = dx * sy / sx;
+	if (abs(dy_new) >= abs(dy)) {
+	    p1.y += (dy_new - dy) / 2;
+	    p2.y -= (dy_new - dy) / 2;
+	} else {
+	    int dx_new = dy * sx / sy;
+	    p1.x += (dx_new - dx) / 2;
+	    p2.x -= (dx_new - dx) / 2;
+	}
+#endif
+    }
+    zoombox.set(p1, p2);
+    ForceRefresh();
+}
+
+void GfxCore::ZoomBoxGo()
+{
+    if (!zoombox.active()) return;
+
+    int width = GetXSize();
+    int height = GetYSize();
+
+    TranslateCave(-0.5 * (zoombox.x1 + zoombox.x2 - width),
+		  -0.5 * (zoombox.y1 + zoombox.y2 - height));
+    int box_w = abs(zoombox.x1 - zoombox.x2);
+    int box_h = abs(zoombox.y1 - zoombox.y2);
+
+    double factor = min(double(width) / box_w, double(height) / box_h);
+
+    zoombox.unset();
+
+    SetScale(GetScale() * factor);
+}
