@@ -26,6 +26,7 @@
 #define gfxcore_h
 
 #include <float.h>
+#include <limits.h>
 #include <time.h>
 
 #include "img_hosted.h"
@@ -88,6 +89,27 @@ class PresentationMark : public Point {
     bool is_valid() const { return scale > 0; }
 };
 
+struct ZoomBox {
+  public:
+    int x1, y1, x2, y2;
+
+    ZoomBox()
+	: x1(INT_MAX) { }
+
+    bool active() const { return x1 != INT_MAX; }
+
+    void set(const wxPoint & p1, const wxPoint & p2) {
+	x1 = p1.x;
+	y1 = p1.y;
+	x2 = p2.x;
+	y2 = p2.y;
+    }
+
+    void unset() {
+	x1 = INT_MAX;
+    }
+};
+
 enum {
     COLOUR_BY_NONE,
     COLOUR_BY_DEPTH,
@@ -181,6 +203,8 @@ private:
     bool m_Metric;
     bool m_Percent;
 
+    bool m_HitTestDebug;
+
     list<LabelInfo*> *m_PointGrid;
     bool m_HitTestGridValid;
 
@@ -204,11 +228,15 @@ private:
 
     cursor current_cursor;
 
+    int sqrd_measure_threshold;
+
     // The legends for each entry in the colour key.
     wxString key_legends[NUM_COLOUR_BANDS];
 
     wxPoint key_lowerleft[COLOUR_BY_LIMIT_];
 
+    ZoomBox zoombox;
+
     void PlaceVertexWithColour(const Vector3 &v, Double factor = 1.0);
     void PlaceVertexWithColour(const Vector3 & v, GLint tex_x, GLint tex_y,
 			       Double factor);
@@ -432,6 +460,9 @@ public:
 	InvalidateList(LIST_DEPTH_KEY);
 	InvalidateList(LIST_SCALE_BAR);
     }
+    void ToggleHitTestDebug() {
+	ToggleFlag(&m_HitTestDebug);
+    }
     void ToggleDegrees() { ToggleFlag(&m_Degrees); }
     void TogglePercent() { ToggleFlag(&m_Percent); }
     void ToggleTubes() { ToggleFlag(&m_Tubes); }
@@ -439,6 +470,7 @@ public:
     void ToggleSmoothShading();
     bool DisplayingBoundingBox() const { return m_BoundingBox; }
     void ToggleBoundingBox() { ToggleFlag(&m_BoundingBox); }
+    void ToggleFatFinger();
 
     bool GetMetric() const { return m_Metric; }
     bool GetDegrees() const { return m_Degrees; }
@@ -490,9 +522,12 @@ public:
     void SetColourBy(int colour_by);
     bool ExportMovie(const wxString & fnm);
     void OnPrint(const wxString &filename, const wxString &title,
-		 const wxString &datestamp, time_t datestamp_numeric);
+		 const wxString &datestamp, time_t datestamp_numeric,
+		 const wxString &cs_proj,
+		 bool close_after_print = false);
     void OnExport(const wxString &filename, const wxString &title,
-		  const wxString &datestamp, time_t datestamp_numeric);
+		  const wxString &datestamp, time_t datestamp_numeric,
+		  const wxString &cs_proj);
     void SetCursor(GfxCore::cursor new_cursor);
     bool MeasuringLineActive() const;
 
@@ -503,6 +538,17 @@ public:
 	    InvalidateList(i);
 	}
     }
+
+    void SetZoomBox(wxPoint p1, wxPoint p2, bool centred, bool aspect);
+
+    void UnsetZoomBox() {
+	if (!zoombox.active()) return;
+	zoombox.unset();
+	ForceRefresh();
+    }
+
+    void ZoomBoxGo();
+
 private:
     DECLARE_EVENT_TABLE()
 };
