@@ -1,6 +1,6 @@
 /* img.c
  * Routines for reading and writing Survex ".3d" image files
- * Copyright (C) 1993-2004,2005,2006,2010,2011,2013 Olly Betts
+ * Copyright (C) 1993-2004,2005,2006,2010,2011,2013,2014 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,28 +21,38 @@
 # include <config.h>
 #endif
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
-#include <time.h>
 #include <ctype.h>
+#include <errno.h>
+#include <limits.h>
 #include <locale.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
 
 #include "img.h"
 
 #define TIMENA "?"
 #ifdef IMG_HOSTED
+# define INT32_T int32_t
 # include "debug.h"
 # include "filelist.h"
 # include "filename.h"
 # include "message.h"
 # include "useful.h"
 # define TIMEFMT msg(/*%a,%Y.%m.%d %H:%M:%S %Z*/107)
-# ifndef INT32_T
-#  define INT32_T int32_t
-# endif
 #else
-# define INT32_T int
+# ifdef HAVE_STDINT_H
+#  include <stdint.h>
+#  define INT32_T int32_t
+# else
+#  include <limits.h>
+#  if INT_MAX >= 2147483647
+#   define INT32_T int
+#  else
+#   define INT32_T long
+#  endif
+# endif
 # define TIMEFMT "%a,%Y.%m.%d %H:%M:%S %Z"
 # define EXT_SVX_3D "3d"
 # define EXT_SVX_POS "pos"
@@ -59,8 +69,12 @@
 /* path isn't used in img.c, but EXT is */
 # define fopenWithPthAndExt(PTH,FNM,EXT,MODE,X) \
     ((*(X) = NULL), fopen(FNM,MODE))
-# define PUTC(C, FH) putc(C, FH)
-# define GETC(FH) getc(FH)
+# ifndef PUTC
+#  define PUTC(C, FH) putc(C, FH)
+# endif
+# ifndef GETC
+#  define GETC(FH) getc(FH)
+# endif
 # define fputsnl(S, FH) (fputs((S), (FH)) == EOF ? EOF : putc('\n', (FH)))
 # define SVX_ASSERT(X)
 
@@ -139,6 +153,85 @@ baseleaf_from_fnm(const char *fnm)
 }
 #endif
 
+static char * my_strdup(const char *str);
+
+static time_t
+mktime_with_tz(struct tm * tm, const char * tz)
+{
+    time_t r;
+    char * old_tz = getenv("TZ");
+#ifdef _MSC_VER
+    if (old_tz) {
+	old_tz = my_strdup(old_tz);
+	if (!old_tz)
+	    return (time_t)-1;
+    }
+    if (_putenv_s("TZ", tz) != 0) {
+	osfree(old_tz);
+	return (time_t)-1;
+    }
+#elif defined HAVE_SETENV
+    if (old_tz) {
+	old_tz = my_strdup(old_tz);
+	if (!old_tz)
+	    return (time_t)-1;
+    }
+    if (setenv("TZ", tz, 1) < 0) {
+	osfree(old_tz);
+	return (time_t)-1;
+    }
+#else
+    char * p;
+    if (old_tz) {
+	size_t len = strlen(old_tz) + 1;
+	p = (char *)xosmalloc(len + 3);
+	if (!p)
+	    return (time_t)-1;
+	memcpy(p, "TZ=", 3);
+	memcpy(p + 3, tz, len);
+	old_tz = p;
+    }
+    p = (char *)xosmalloc(strlen(tz) + 4);
+    if (!p) {
+	osfree(old_tz);
+	return (time_t)-1;
+    }
+    memcpy(p, "TZ=", 3);
+    strcpy(p + 3, tz);
+    if (putenv(p) != 0) {
+	osfree(p);
+	osfree(old_tz);
+	return (time_t)-1;
+    }
+#define CLEANUP() osfree(p)
+#endif
+    tzset();
+    r = mktime(tm);
+    if (old_tz) {
+#ifdef _MSC_VER
+	_putenv_s("TZ", old_tz);
+#elif !defined HAVE_SETENV
+	putenv(old_tz);
+#else
+	setenv("TZ", old_tz, 1);
+#endif
+	osfree(old_tz);
+    } else {
+#ifdef _MSC_VER
+	_putenv_s("TZ", "");
+#elif !defined HAVE_UNSETENV
+	putenv((char*)"TZ");
+#else
+	unsetenv("TZ");
+#endif
+    }
+#ifdef CLEANUP
+    CLEANUP();
+#undef CLEANUP
+#endif
+    return r;
+}
+
 static unsigned short
 getu16(FILE *fh)
 {
@@ -175,9 +268,8 @@ my_lround(double x) {
 #if defined(strcasecmp) || defined(HAVE_STRCASECMP)
 # define my_strcasecmp strcasecmp
 #else
-/* What about top bit set chars? */
-int my_strcasecmp(const char *s1, const char *s2) {
-   register int c1, c2;
+static int my_strcasecmp(const char *s1, const char *s2) {
+   unsigned char c1, c2;
    do {
       c1 = *s1++;
       c2 = *s2++;
@@ -195,7 +287,6 @@ static img_errcode img_errno = IMG_NONE;
 
 #define EXT_PLT "plt"
 #define EXT_PLF "plf"
-#define EXT_XYZ "xyz"
 
 /* Attempt to string paste to ensure we are passed a literal string */
 #define LITLEN(S) (sizeof(S"") - 1)
@@ -216,8 +307,10 @@ my_strdup(const char *str)
    return p;
 }
 
+#define getline_alloc(FH) getline_alloc_len(FH, NULL)
+
 static char *
-getline_alloc(FILE *fh)
+getline_alloc_len(FILE *fh, size_t * p_len)
 {
    int ch;
    size_t i = 0;
@@ -246,7 +339,8 @@ getline_alloc(FILE *fh)
       /* if it's not the other eol character, put it back */
       if (ch != otherone) ungetc(ch, fh);
    }
-   buf[i++] = '\0';
+   buf[i] = '\0';
+   if (p_len) *p_len = i;
    return buf;
 }
 
@@ -335,7 +429,9 @@ img_open_survey(const char *fnm, const char *survey)
 
    pimg->l = pimg->r = pimg->u = pimg->d = -1.0;
 
-   pimg->title = pimg->datestamp = NULL;
+   pimg->title = pimg->datestamp = pimg->cs = NULL;
+   pimg->datestamp_numeric = (time_t)-1;
+
    if (survey) {
       len = strlen(survey);
       if (len) {
@@ -458,7 +554,16 @@ plt_file:
       }
    }
 
-   if (has_ext(fnm, len, EXT_XYZ)) {
+   /* Although these are often referred to as "CMAP .XYZ files", it seems
+    * that actually, the extension .XYZ isn't used, rather .SHT (shot
+    * variant, produced by CMAP v16 and later), .UNA (unadjusted) and
+    * .ADJ (adjusted) extensions are.  Since img has long checked for
+    * .XYZ, we continue to do so in case anyone is relying on it.
+    */
+   if (has_ext(fnm, len, "sht") ||
+       has_ext(fnm, len, "adj") ||
+       has_ext(fnm, len, "una") ||
+       has_ext(fnm, len, "xyz")) {
       char *line;
 xyz_file:
       /* Spaces aren't legal in CMAP station names, but dots are, so
@@ -469,14 +574,78 @@ xyz_file:
 	 img_errno = IMG_OUTOFMEMORY;
 	 goto error;
       }
-      /* FIXME: reparse date? */
+      /* There doesn't seem to be a spec for what happens after 1999 with cmap
+       * files, so this code allows for:
+       *  * 21xx -> xx (up to 2150)
+       *  * 21xx -> 1xx (up to 2199)
+       *  * full year being specified instead of 2 digits
+       */
       len = strlen(line);
-      if (len > 59) line[59] = '\0';
+      if (len > 59) {
+	 /* Don't just truncate at column 59, allow for a > 2 digit year. */
+	 char * p = strstr(line + len, "Page");
+	 if (p) {
+	    while (p > line && p[-1] == ' ')
+	       --p;
+	    *p = '\0';
+	    len = p - line;
+	 } else {
+	    line[59] = '\0';
+	 }
+      }
       if (len > 45) {
+	 /* YY/MM/DD HH:MM */
+	 struct tm tm;
+	 unsigned long v;
+	 char * p;
 	 pimg->datestamp = my_strdup(line + 45);
+	 p = pimg->datestamp;
+	 v = strtoul(p, &p, 10);
+	 if (v <= 50) {
+	    /* In the absence of a spec for cmap files, assume <= 50 means 21st
+	     * century. */
+	    v += 2000;
+	 } else if (v < 200) {
+	    /* Map 100-199 to 21st century. */
+	    v += 1900;
+	 }
+	 if (v == ULONG_MAX || *p++ != '/')
+	    goto bad_cmap_date;
+	 tm.tm_year = v - 1900;
+	 v = strtoul(p, &p, 10);
+	 if (v < 1 || v > 12 || *p++ != '/')
+	    goto bad_cmap_date;
+	 tm.tm_mon = v - 1;
+	 v = strtoul(p, &p, 10);
+	 if (v < 1 || v > 31 || *p++ != ' ')
+	    goto bad_cmap_date;
+	 tm.tm_mday = v;
+	 v = strtoul(p, &p, 10);
+	 if (v >= 24 || *p++ != ':')
+	    goto bad_cmap_date;
+	 tm.tm_hour = v;
+	 v = strtoul(p, &p, 10);
+	 if (v >= 60)
+	    goto bad_cmap_date;
+	 tm.tm_min = v;
+	 if (*p == ':') {
+	    v = strtoul(p + 1, &p, 10);
+	    if (v > 60)
+	       goto bad_cmap_date;
+	    tm.tm_sec = v;
+	 } else {
+	    tm.tm_sec = 0;
+	 }
+	 tm.tm_isdst = 0;
+	 /* We have no indication of what timezone this timestamp is in.  It's
+	  * probably local time for whoever processed the data, so just assume
+	  * UTC, which is at least fairly central in the possibilities.
+	  */
+	 pimg->datestamp_numeric = mktime_with_tz(&tm, "");
       } else {
 	 pimg->datestamp = my_strdup(TIMENA);
       }
+bad_cmap_date:
       if (strncmp(line, "  Cave Survey Data Processed by CMAP ",
 		  LITLEN("  Cave Survey Data Processed by CMAP ")) == 0) {
 	 len = 0;
@@ -584,15 +753,32 @@ xyz_file:
    }
 
 v03d:
-   if (!pimg->title)
-       pimg->title = getline_alloc(pimg->fh);
-   else
-       osfree(getline_alloc(pimg->fh));
+   {
+       size_t title_len;
+       char * title = getline_alloc_len(pimg->fh, &title_len);
+       if (pimg->version == 8 && title) {
+	   /* We sneak in an extra field after a zero byte here, containing the
+	    * specified coordinate system (if any).  Older readers will just
+	    * not see it (which is fine), and this trick avoids us having to
+	    * bump the 3d format version.
+	    */
+	   size_t real_len = strlen(title);
+	   if (real_len != title_len) {
+	       pimg->cs = my_strdup(title + real_len + 1);
+	   }
+       }
+       if (!pimg->title) {
+	   pimg->title = title;
+       } else {
+	   osfree(title);
+       }
+   }
    pimg->datestamp = getline_alloc(pimg->fh);
    if (!pimg->title || !pimg->datestamp) {
       img_errno = IMG_OUTOFMEMORY;
       error:
       osfree(pimg->title);
+      osfree(pimg->cs);
       osfree(pimg->datestamp);
       osfree(pimg->filename_opened);
       fclose(pimg->fh);
@@ -611,6 +797,62 @@ v03d:
       }
    }
 
+   if (pimg->datestamp[0] == '@') {
+      unsigned long v;
+      char * p;
+      errno = 0;
+      v = strtoul(pimg->datestamp + 1, &p, 10);
+      if (errno == 0 && *p == '\0')
+	 pimg->datestamp_numeric = v;
+      /* FIXME: We're assuming here that the C time_t epoch is 1970, which is
+       * true for Unix-like systems, Mac OS X and Windows, but isn't guaranteed
+       * by ISO C.
+       */
+   } else {
+      /* %a,%Y.%m.%d %H:%M:%S %Z */
+      struct tm tm;
+      unsigned long v;
+      char * p = pimg->datestamp;
+      while (isalpha((unsigned char)*p)) ++p;
+      if (*p == ',') ++p;
+      while (isspace((unsigned char)*p)) ++p;
+      v = strtoul(p, &p, 10);
+      if (v == ULONG_MAX || *p++ != '.')
+	 goto bad_3d_date;
+      tm.tm_year = v - 1900;
+      v = strtoul(p, &p, 10);
+      if (v < 1 || v > 12 || *p++ != '.')
+	 goto bad_3d_date;
+      tm.tm_mon = v - 1;
+      v = strtoul(p, &p, 10);
+      if (v < 1 || v > 31 || *p++ != ' ')
+	 goto bad_3d_date;
+      tm.tm_mday = v;
+      v = strtoul(p, &p, 10);
+      if (v >= 24 || *p++ != ':')
+	 goto bad_3d_date;
+      tm.tm_hour = v;
+      v = strtoul(p, &p, 10);
+      if (v >= 60 || *p++ != ':')
+	 goto bad_3d_date;
+      tm.tm_min = v;
+      v = strtoul(p, &p, 10);
+      if (v > 60)
+	 goto bad_3d_date;
+      tm.tm_sec = v;
+      tm.tm_isdst = 0;
+      while (isspace((unsigned char)*p)) ++p;
+      /* p now points to the timezone string.
+       *
+       * However, it's likely to be a string like "BST", and such strings can
+       * be ambiguous (BST could be UTC+1 or UTC+6), so it is impossible to
+       * reliably convert in all cases.  Just pass what we have to tzset() - if
+       * it doesn't handle it, UTC will be used.
+       */
+      pimg->datestamp_numeric = mktime_with_tz(&tm, p);
+   }
+bad_3d_date:
+
    pimg->start = ftell(pimg->fh);
 
    return pimg;
@@ -646,7 +888,7 @@ img_rewind(img *pimg)
 }
 
 img *
-img_open_write(const char *fnm, char *title, int flags)
+img_open_write_cs(const char *fnm, const char *title, const char * cs, int flags)
 {
    time_t tm;
    img *pimg;
@@ -698,6 +940,15 @@ img_open_write(const char *fnm, char *title, int flags)
       if (len < 11 || strcmp(title + len - 11, " (extended)") != 0)
 	 fputs(" (extended)", pimg->fh);
    }
+   if (pimg->version == 8 && cs && *cs) {
+      /* We sneak in an extra field after a zero byte here, containing the
+       * specified coordinate system (if any).  Older readers will just not
+       * see it (which is fine), and this trick avoids us having to bump the
+       * 3d format version.
+       */
+      PUTC('\0', pimg->fh);
+      fputs(cs, pimg->fh);
+   }
    PUTC('\n', pimg->fh);
 
    tm = time(NULL);
@@ -951,7 +1202,7 @@ img_read_item(img *pimg, img_point *p)
    }
 }
 
-int
+static int
 img_read_item_new(img *pimg, img_point *p)
 {
    int result;
@@ -1130,7 +1381,7 @@ img_read_item_new(img *pimg, img_point *p)
    return result;
 }
 
-int
+static int
 img_read_item_v3to7(img *pimg, img_point *p)
 {
    int result;
@@ -1237,6 +1488,14 @@ img_read_item_v3to7(img *pimg, img_point *p)
 		  pimg->E = get32(pimg->fh) / 100.0;
 		  pimg->H = get32(pimg->fh) / 100.0;
 		  pimg->V = get32(pimg->fh) / 100.0;
+		  if (feof(pimg->fh)) {
+		      img_errno = IMG_BADFORMAT;
+		      return img_BAD;
+		  }
+		  if (ferror(pimg->fh)) {
+		      img_errno = IMG_READERROR;
+		      return img_BAD;
+		  }
 		  return img_ERROR_INFO;
 	      case 0x23: { /* v7+: Date range (long) */
 		  if (pimg->version < 7) {
@@ -1245,6 +1504,14 @@ img_read_item_v3to7(img *pimg, img_point *p)
 		  }
 		  int days1 = (int)getu16(pimg->fh);
 		  int days2 = (int)getu16(pimg->fh);
+		  if (feof(pimg->fh)) {
+		      img_errno = IMG_BADFORMAT;
+		      return img_BAD;
+		  }
+		  if (ferror(pimg->fh)) {
+		      img_errno = IMG_READERROR;
+		      return img_BAD;
+		  }
 #if IMG_API_VERSION == 0
 		  pimg->date1 = (days1 - 25567) * 86400;
 		  pimg->date2 = (days2 - 25567) * 86400;
@@ -1277,6 +1544,14 @@ img_read_item_v3to7(img *pimg, img_point *p)
 		      pimg->u = get32(pimg->fh) / 100.0;
 		      pimg->d = get32(pimg->fh) / 100.0;
 		  }
+		  if (feof(pimg->fh)) {
+		      img_errno = IMG_BADFORMAT;
+		      return img_BAD;
+		  }
+		  if (ferror(pimg->fh)) {
+		      img_errno = IMG_READERROR;
+		      return img_BAD;
+		  }
 		  if (pimg->survey_len) {
 		      size_t l = pimg->survey_len;
 		      const char *s = pimg->label_buf;
@@ -1298,6 +1573,14 @@ img_read_item_v3to7(img *pimg, img_point *p)
 		  img_errno = IMG_BADFORMAT;
 		  return img_BAD;
 	  }
+	  if (feof(pimg->fh)) {
+	      img_errno = IMG_BADFORMAT;
+	      return img_BAD;
+	  }
+	  if (ferror(pimg->fh)) {
+	      img_errno = IMG_READERROR;
+	      return img_BAD;
+	  }
 	  goto again3;
       }
       /* 16-31 mean remove (n - 15) characters from the prefix */
@@ -1364,7 +1647,7 @@ img_read_item_v3to7(img *pimg, img_point *p)
    return result;
 }
 
-int
+static int
 img_read_item_ancient(img *pimg, img_point *p)
 {
    int result;
@@ -1372,9 +1655,8 @@ img_read_item_ancient(img *pimg, img_point *p)
    static img_point pt = { 0.0, 0.0, 0.0 };
    long opt;
 
-   pimg->label = pimg->label_buf;
-
    again: /* label to goto if we get a cross */
+   pimg->label = pimg->label_buf;
    pimg->label[0] = '\0';
 
    if (pimg->version == 1) {
@@ -1408,32 +1690,21 @@ img_read_item_ancient(img *pimg, img_point *p)
       }
       goto again;
     case 2: case 3: {
-      char *q;
-      int ch;
+      size_t len;
       result = img_LABEL;
-      ch = GETC(pimg->fh);
-      if (ch == EOF) {
+      if (!fgets(pimg->label_buf, pimg->buf_len, pimg->fh)) {
 	 img_errno = feof(pimg->fh) ? IMG_BADFORMAT : IMG_READERROR;
 	 return img_BAD;
       }
-      if (ch != '\\') ungetc(ch, pimg->fh);
-      fgets(pimg->label_buf, 257, pimg->fh);
-      if (feof(pimg->fh)) {
-	 img_errno = IMG_BADFORMAT;
-	 return img_BAD;
-      }
-      if (ferror(pimg->fh)) {
-	 img_errno = IMG_READERROR;
-	 return img_BAD;
-      }
-      q = pimg->label_buf + strlen(pimg->label_buf) - 1;
-      if (*q != '\n') {
+      if (pimg->label[0] == '\\') pimg->label++;
+      len = strlen(pimg->label);
+      if (len == 0 || pimg->label[len - 1] != '\n') {
 	 img_errno = IMG_BADFORMAT;
 	 return img_BAD;
       }
       /* Ignore empty labels in some .3d files (caused by a bug) */
-      if (q == pimg->label_buf) goto again;
-      *q = '\0';
+      if (len == 1) goto again;
+      pimg->label[len - 1] = '\0';
       pimg->flags = img_SFLAG_UNDERGROUND; /* no flags given... */
       if (opt == 2) goto done;
       break;
@@ -1487,7 +1758,7 @@ img_read_item_ancient(img *pimg, img_point *p)
 	 char *q;
 	 pimg->flags = (int)opt & 0x3f;
 	 result = img_LABEL;
-	 if (!fgets(pimg->label_buf, 257, pimg->fh)) {
+	 if (!fgets(pimg->label_buf, pimg->buf_len, pimg->fh)) {
 	    img_errno = feof(pimg->fh) ? IMG_BADFORMAT : IMG_READERROR;
 	    return img_BAD;
 	 }
@@ -1501,8 +1772,6 @@ img_read_item_ancient(img *pimg, img_point *p)
 	 *q = '\0';
 	 break;
        }
-       case 0xc0:
-	 /* use this for an extra leg or station flag if we need it */
        default:
 	 img_errno = IMG_BADFORMAT;
 	 return img_BAD;
@@ -1513,7 +1782,7 @@ img_read_item_ancient(img *pimg, img_point *p)
    if (!read_coord(pimg->fh, &pt)) return img_BAD;
 
    if (result == img_LABEL && pimg->survey_len) {
-      if (strncmp(pimg->label_buf, pimg->survey, pimg->survey_len + 1) != 0)
+      if (strncmp(pimg->label, pimg->survey, pimg->survey_len + 1) != 0)
 	 goto again;
       pimg->label += pimg->survey_len + 1;
    }
@@ -1534,7 +1803,7 @@ img_read_item_ancient(img *pimg, img_point *p)
 	 return img_BAD;
       }
 
-      if (opt_lookahead == 2) return img_read_item(pimg, p);
+      if (opt_lookahead == 2) return img_read_item_ancient(pimg, p);
    }
 
    return result;
@@ -1912,7 +2181,7 @@ skip_to_N:
 	 /* FIXME: duplicate stations... */
 	 return img_LABEL;
       } else {
-	 /* Shot variant */
+	 /* Shot variant (VERSION_CMAP_SHOT) */
 	 char old[8], new_[8];
 	 if (len < 61) {
 	    osfree(line);
@@ -2383,7 +2652,7 @@ img_write_item_ancient(img *pimg, int code, int flags, const char *s,
     case img_LABEL:
       if (pimg->version == 1) {
 	 /* put a move before each label */
-	 img_write_item(pimg, img_MOVE, 0, NULL, x, y, z);
+	 img_write_item_ancient(pimg, img_MOVE, 0, NULL, x, y, z);
 	 put32(2, pimg->fh);
 	 fputsnl(s, pimg->fh);
 	 return;
@@ -2452,6 +2721,7 @@ img_close(img *pimg)
 	 if (pimg->fRead) {
 	    osfree(pimg->survey);
 	    osfree(pimg->title);
+	    osfree(pimg->cs);
 	    osfree(pimg->datestamp);
 	 } else {
 	    /* write end of data marker */
