@@ -1,6 +1,6 @@
 /* img.h
  * Header file for routines to read and write Survex ".3d" image files
- * Copyright (C) Olly Betts 1993,1994,1997,2001,2002,2003,2004,2005,2006,2010,2011,2012,2013
+ * Copyright (C) Olly Betts 1993,1994,1997,2001,2002,2003,2004,2005,2006,2010,2011,2012,2013,2014
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -91,6 +91,10 @@ typedef struct {
    char *label;
    int flags;
    char *title;
+   /* If the coordinate system was specified, this contains a PROJ4 string
+    * describing it.  If not, this member will be NULL.
+    */
+   char *cs;
    /* Older .3d format versions stored a human readable datestamp string.
     * Format versions >= 8 versions store a string consisting of "@" followed
     * by the number of seconds since midnight UTC on 1/1/1970.  Some foreign
@@ -98,6 +102,20 @@ typedef struct {
     * (which results in "?" being returned).
     */
    char *datestamp;
+   /* The datestamp as a time_t (or (time_t)-1 if not available).
+    *
+    * For 3d format versions >= 8, this is a reliable value and in UTC.  Older
+    * 3d format versions store a human readable time, which img will attempt
+    * to decode, but it may fail, particularly with handling timezones.  Even
+    * if it does work, beware that times in local time where DST applies are
+    * inherently ambiguous around when the clocks go back.
+    *
+    * CMAP XYZ files contain a timestamp.  It's probably in localtime (but
+    * without any timezone information) and the example files are all pre-2000
+    * and have two digit years.  We do our best to turn these into a useful
+    * time_t value.
+    */
+   time_t datestamp_numeric;
    char separator; /* character used to separate survey levels ('.' usually) */
 #if IMG_API_VERSION == 0
    time_t date1, date2;
@@ -185,7 +203,20 @@ img *img_open_survey(const char *fnm, const char *survey);
  * Returns pointer to an img struct or NULL for error (check img_error()
  * for details)
  */
-img *img_open_write(const char *fnm, char *title, int flags);
+#define img_open_write(F, T, S) img_open_write_cs(F, T, NULL, S)
+
+/* Open a .3d file for output in a specified coordinate system
+ * fnm is the filename
+ * title is the title
+ * cs is a PROJ4 string describing the coordinate system (or NULL)
+ * flags contains a bitwise-or of any file-wide flags - currently only one
+ * is available: img_FFLAG_EXTENDED.
+ *
+ * Returns pointer to an img struct or NULL for error (check img_error()
+ * for details)
+ */
+img *img_open_write_cs(const char *fnm, const char *title, const char * cs,
+		       int flags);
 
 /* Read an item from a .3d file
  * pimg is a pointer to an img struct returned by img_open()
