@@ -243,7 +243,7 @@ class MainFrm : public wxFrame {
     wxNotebook* m_Notebook;
     AvenPresList* m_PresList;
     wxString m_File;
-    wxString m_Title, m_DateStamp;
+    wxString m_Title, m_cs_proj, m_DateStamp;
     time_t m_DateStamp_numeric;
     wxChar separator; // character separating survey levels (often '.')
     Vector3 m_Offsets;
@@ -262,7 +262,7 @@ class MainFrm : public wxFrame {
 
     void FillTree();
     bool ProcessSVXFile(const wxString & file);
-    bool LoadData(const wxString& file, wxString prefix = wxString());
+    bool LoadData(const wxString& file, const wxString& prefix);
 //    void FixLRUD(traverse & centreline);
     void CentreDataset(const Vector3 & vmin);
 
@@ -280,7 +280,7 @@ public:
     void InitialiseAfterLoad(const wxString & file);
 
     void OnMRUFile(wxCommandEvent& event);
-    void OpenFile(const wxString& file, wxString survey = wxString());
+    void OpenFile(const wxString& file, const wxString& survey = wxString());
 
     void OnPresNewUpdate(wxUpdateUIEvent& event);
     void OnPresOpenUpdate(wxUpdateUIEvent& event);
@@ -311,6 +311,7 @@ public:
     void OnFilePreferences(wxCommandEvent& event);
     void OnFileOpenTerrain(wxCommandEvent& event);
     void OnPrint(wxCommandEvent& event);
+    void PrintAndExit();
     void OnPageSetup(wxCommandEvent& event);
     void OnPresNew(wxCommandEvent& event);
     void OnPresOpen(wxCommandEvent& event);
