@@ -77,6 +77,7 @@ enum {
 	svx_ENTS,
 	svx_FIXES,
 	svx_EXPORTS,
+	svx_PROJ_LABEL,
 	svx_PROJ,
 	svx_GRID,
 	svx_TEXT_HEIGHT,
@@ -192,6 +193,7 @@ BEGIN_EVENT_TABLE(svxPrintDlg, wxDialog)
     EVT_SPINCTRL(svx_TILT, svxPrintDlg::OnChangeSpin)
     EVT_BUTTON(wxID_PRINT, svxPrintDlg::OnPrint)
     EVT_BUTTON(svx_EXPORT, svxPrintDlg::OnExport)
+    EVT_BUTTON(wxID_CANCEL, svxPrintDlg::OnCancel)
 #ifdef AVEN_PRINT_PREVIEW
     EVT_BUTTON(wxID_PREVIEW, svxPrintDlg::OnPreview)
 #endif
@@ -248,7 +250,7 @@ static wxString projs[] = {
 static unsigned format_info[] = {
     LABELS|LEGS|SURF|STNS|PASG|XSECT|WALLS|MARKER_SIZE|TEXT_HEIGHT|GRID|FULL_COORDS,
     LABELS|LEGS|SURF|STNS,
-    LABELS|LEGS|SURF|ENTS|FIXES|EXPORTS/*|PROJ*/|EXPORT_3D,
+    LABELS|LEGS|SURF|ENTS|FIXES|EXPORTS|PROJ|EXPORT_3D,
     LABELS|LEGS|SURF|STNS|CENTRED,
     LABELS|LEGS|SURF,
     LABELS|LEGS|SURF|STNS|MARKER_SIZE|GRID|SCALE,
@@ -278,15 +280,21 @@ static int msg_filetype[] = {
 // there are three jobs to do here...
 // User <-> wx - this should possibly be done in a separate file
 svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
-			 const wxString & title,
+			 const wxString & title, const wxString & cs_proj,
 			 const wxString & datestamp, time_t datestamp_numeric,
 			 double angle, double tilt_angle,
 			 bool labels, bool crosses, bool legs, bool surf,
 			 bool tubes, bool ents, bool fixes, bool exports,
-			 bool printing)
-	: wxDialog(mainfrm_, -1, wxString(printing ? wmsg(/*Print*/399) : wmsg(/*Export*/383))),
+			 bool printing, bool close_after_)
+	: wxDialog(mainfrm_, -1, wxString(printing ?
+					  /* TRANSLATORS: Title of the print
+					   * dialog */
+					  wmsg(/*Print*/399) :
+					  /* TRANSLATORS: Title of the export
+					   * dialog */
+					  wmsg(/*Export*/383))),
 	  m_layout(printing ? wxGetApp().GetPageSetupDialogData() : NULL),
-	  m_File(filename), mainfrm(mainfrm_)
+	  m_File(filename), mainfrm(mainfrm_), close_after(close_after_)
 {
     m_scale = NULL;
     m_printSize = NULL;
@@ -315,6 +323,7 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
     m_layout.datestamp_numeric = datestamp_numeric;
     m_layout.rot = int(angle);
     m_layout.title = title;
+    m_layout.cs_proj = cs_proj;
     if (mainfrm->IsExtendedElevation()) {
 	m_layout.view = layout::EXTELEV;
 	if (m_layout.rot != 0 && m_layout.rot != 180) m_layout.rot = 0;
@@ -334,11 +343,16 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
     /* setup our print dialog*/
     wxBoxSizer* v1 = new wxBoxSizer(wxVERTICAL);
     wxBoxSizer* h1 = new wxBoxSizer(wxHORIZONTAL); // holds controls
+    /* TRANSLATORS: Used as a label for the surrounding box for the "Bearing"
+     * and "Tilt angle" fields, and the "Plan view" and "Elevation" buttons in
+     * the "what to print/export" dialog. */
     m_viewbox = new wxStaticBoxSizer(new wxStaticBox(this, -1, wmsg(/*View*/283)), wxVERTICAL);
+    /* TRANSLATORS: Used as a label for the surrounding box for the "survey
+     * legs" "stations" "names" etc checkboxes in the "what to print" dialog.
+     * "Elements" isn’t a good name for this but nothing better has yet come to
+     * mind! */
     wxBoxSizer* v3 = new wxStaticBoxSizer(new wxStaticBox(this, -1, wmsg(/*Elements*/256)), wxVERTICAL);
-#if 0
     wxBoxSizer* h2 = new wxBoxSizer(wxHORIZONTAL);
-#endif
     wxBoxSizer* h3 = new wxBoxSizer(wxHORIZONTAL); // holds buttons
 
     if (!printing) {
@@ -371,6 +385,9 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
     label = new wxStaticText(this, -1, wxString(wmsg(/*Scale*/154)) + wxT(" 1:"));
     if (scales[0].empty()) {
 	if (printing) {
+	    /* TRANSLATORS: used in the scale drop down selector in the print
+	     * dialog the implicit meaning is "choose a suitable scale to fit
+	     * the plot on a single page", but we need something shorter */
 	    scales[0].assign(wmsg(/*One page*/258));
 	} else {
 	    scales[0].assign(wxT("1000"));
@@ -407,6 +424,7 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	m_bearing = new wxSpinCtrl(this, svx_BEARING);
 	m_bearing->SetRange(0, 359);
 	anglebox->Add(m_bearing, 0, wxALIGN_CENTER|wxALL, 5);
+	/* TRANSLATORS: Used in the print dialog: */
 	tilt_label = new wxStaticText(this, -1, wmsg(/*Tilt angle*/263));
 	anglebox->Add(tilt_label, 0, wxALIGN_CENTER_VERTICAL|wxALIGN_LEFT|wxALL, 5);
 	m_tilt = new wxSpinCtrl(this, svx_TILT);
@@ -424,10 +442,14 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	m_viewbox->Add(planelevsizer, 0, wxALIGN_LEFT|wxALL, 5);
     }
 
+    /* TRANSLATORS: Here a "survey leg" is a set of measurements between two
+     * "survey stations". */
     v3->Add(new wxCheckBox(this, svx_LEGS, wmsg(/*Underground Survey Legs*/262),
 			   wxDefaultPosition, wxDefaultSize, 0,
 			   BitValidator(&m_layout.show_mask, LEGS)),
 	    0, wxALIGN_LEFT|wxALL, 2);
+    /* TRANSLATORS: Here a "survey leg" is a set of measurements between two
+     * "survey stations". */
     v3->Add(new wxCheckBox(this, svx_SURFACE, wmsg(/*Sur&face Survey Legs*/403),
 			   wxDefaultPosition, wxDefaultSize, 0,
 			   BitValidator(&m_layout.show_mask, SURF)),
@@ -475,12 +497,19 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 		0, wxALIGN_LEFT|wxALL, 2);
     }
     if (printing) {
+	/* TRANSLATORS: used in the print dialog - controls drawing lines
+	 * around each page */
 	v3->Add(new wxCheckBox(this, svx_BORDERS, wmsg(/*Page Borders*/264),
 			       wxDefaultPosition, wxDefaultSize, 0,
 			       wxGenericValidator(&m_layout.Border)),
 		0, wxALIGN_LEFT|wxALL, 2);
+	/* TRANSLATORS: will be used in the print dialog - check this to print
+	 * blank pages (otherwise they’ll be skipped to save paper) */
 //	m_blanks = new wxCheckBox(this, svx_BLANKS, wmsg(/*Blank Pages*/266));
 //	v3->Add(m_blanks, 0, wxALIGN_LEFT|wxALL, 2);
+	/* TRANSLATORS: As in the legend on a map.  Used in the print dialog -
+	 * controls drawing the box at the lower left with survey name, view
+	 * angles, etc */
 	v3->Add(new wxCheckBox(this, svx_LEGEND, wmsg(/*Legend*/265),
 			       wxDefaultPosition, wxDefaultSize, 0,
 			       wxGenericValidator(&m_layout.Legend)),
@@ -488,18 +517,37 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
     }
 
     h1->Add(v3, 0, wxALIGN_LEFT|wxALL, 5);
-    h1->Add(m_viewbox, 0, wxALIGN_LEFT|wxALL, 5);
+    h1->Add(m_viewbox, 0, wxALIGN_LEFT|wxLEFT, 5);
+
+    /* TRANSLATORS: The PROJ library is used to do coordinate transformations
+     * (https://trac.osgeo.org/proj/) - if the .3d file doesn't contain details
+     * of the coordinate projection in use, the user must specify it here for
+     * export formats which need to know it (e.g. GPX).
+     */
+    h2->Add(new wxStaticText(this, svx_PROJ_LABEL, wmsg(/*Coordinate projection*/440)),
+	    0, wxLEFT|wxALIGN_CENTRE_VERTICAL, 5);
+    long style = 0;
+    if (!m_layout.cs_proj.empty()) {
+	// If the input file specified the coordinate system, don't let the
+	// user mess with it.
+	style = wxTE_READONLY;
+    } else {
+#if 0 // FIXME: Is it a good idea to save this?
+	wxConfigBase * cfg = wxConfigBase::Get();
+	wxString input_projection;
+	cfg->Read(wxT("input_projection"), &input_projection);
+        if (!input_projection.empty())
+	    proj_edit.SetValue(input_projection);
+#endif
+    }
+    wxTextCtrl * proj_edit = new wxTextCtrl(this, svx_PROJ, m_layout.cs_proj,
+					    wxDefaultPosition, wxDefaultSize,
+					    style);
+    h2->Add(proj_edit, 1, wxALL|wxEXPAND|wxALIGN_CENTRE_VERTICAL, 5);
+    v1->Add(h2, 0, wxALIGN_LEFT|wxEXPAND, 5);
 
     v1->Add(h1, 0, wxALIGN_LEFT|wxALL, 5);
 
-#if 0 // FIXME: too wide as-is
-    h2->Add(new wxStaticText(this, -1, wmsg(/*input datum as string to pass to PROJ*/389)));
-    h2->Add(new wxComboBox(this, svx_PROJ, projs[0], wxDefaultPosition,
-			   wxDefaultSize, sizeof(projs) / sizeof(projs[0]),
-			   projs));
-    v1->Add(h2, 0, wxALIGN_LEFT, 5);
-#endif
-
     // When we enable/disable checkboxes in the export dialog, ideally we'd
     // like the dialog to resize, but not sure how to achieve that, so we
     // add a stretchable spacer here so at least the buttons stay in the
@@ -518,6 +566,8 @@ svxPrintDlg::svxPrintDlg(MainFrm* mainfrm_, const wxString & filename,
 	but = new wxButton(this, wxID_PRINT, wmsg(/*&Print…*/400));
 #endif
     } else {
+	/* TRANSLATORS: The text on the action button in the "Export" settings
+	 * dialog */
 	but = new wxButton(this, svx_EXPORT, wmsg(/*&Export…*/230));
     }
     but->SetDefault();
@@ -564,28 +614,27 @@ svxPrintDlg::OnExport(wxCommandEvent&) {
     filespec += wxT("|");
     filespec += wxFileSelectorDefaultWildcardStr;
 
+    /* TRANSLATORS: Title of file dialog to choose name and type of exported
+     * file. */
     wxFileDialog dlg(this, wmsg(/*Export as:*/401), wxString(), leaf,
 		     filespec, wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
     if (dlg.ShowModal() == wxID_OK) {
-#if 0 // FIXME: sort this out
-	wxString input_projection = ((wxComboBox*)FindWindow(svx_PROJ))->GetValue();
-#else
-	wxConfigBase * cfg = wxConfigBase::Get();
-	wxString input_projection;
-	cfg->Read(wxT("input_projection"), &input_projection,
-		  wxString(wxT("+proj=tmerc +lat_0=0 +lon_0=13d20 +k=1 +x_0=0 +y_0=-5200000 +ellps=bessel +towgs84=577.326,90.129,463.919,5.137,1.474,5.297,2.4232")));
-#endif
+	wxString input_projection = ((wxTextCtrl*)FindWindow(svx_PROJ))->GetValue();
 	double grid = 100; // metres
 	double text_height = 0.6;
 	double marker_size = 0.8;
 
-	if (!Export(dlg.GetPath(), m_layout.title,
-		    m_layout.datestamp, m_layout.datestamp_numeric, mainfrm,
-		    m_layout.rot, m_layout.tilt, m_layout.show_mask,
-		    export_format(format_idx), input_projection.mb_str(),
-		    grid, text_height, marker_size)) {
-	    wxString m = wxString::Format(wmsg(/*Couldn’t write file “%s”*/402).c_str(),
-					  m_File.c_str());
+	try {
+	    if (!Export(dlg.GetPath(), m_layout.title,
+			m_layout.datestamp, m_layout.datestamp_numeric, mainfrm,
+			m_layout.rot, m_layout.tilt, m_layout.show_mask,
+			export_format(format_idx), input_projection.mb_str(),
+			grid, text_height, marker_size)) {
+		wxString m = wxString::Format(wmsg(/*Couldn’t write file “%s”*/402).c_str(),
+					      m_File.c_str());
+		wxGetApp().ReportError(m);
+	    }
+	} catch (const wxString & m) {
 	    wxGetApp().ReportError(m);
 	}
     }
@@ -603,6 +652,7 @@ svxPrintDlg::OnPreview(wxCommandEvent&) {
     pv = new wxPrintPreview(new svxPrintout(mainfrm, &m_layout, psdd, m_File),
 			    new svxPrintout(mainfrm, &m_layout, psdd, m_File),
 			    &pd);
+    // TRANSLATORS: Title of the print preview dialog
     wxPreviewFrame *frame = new wxPreviewFrame(pv, mainfrm, wmsg(/*Print Preview*/398));
     frame->Initialize();
 
@@ -671,6 +721,13 @@ svxPrintDlg::OnChange(wxCommandEvent& e) {
     SomethingChanged(e.GetId());
 }
 
+void
+svxPrintDlg::OnCancel(wxCommandEvent&) {
+    if (close_after)
+	mainfrm->Close();
+    Destroy();
+}
+
 void
 svxPrintDlg::SomethingChanged(int control_id) {
     if ((control_id == 0 || control_id == svx_FORMAT) && m_format) {
@@ -691,9 +748,8 @@ svxPrintDlg::SomethingChanged(int control_id) {
 		{ svx_EXPORTS, EXPORTS },
 		{ svx_CENTRED, CENTRED },
 		{ svx_FULLCOORDS, FULL_COORDS },
-#if 0
+		{ svx_PROJ_LABEL, PROJ },
 		{ svx_PROJ, PROJ },
-#endif
 	    };
 	    static unsigned n_controls = sizeof(controls) / sizeof(controls[0]);
 	    for (unsigned i = 0; i != n_controls; ++i) {
@@ -898,7 +954,7 @@ svxPrintout::draw_info_box()
 
       long xc = boxwidth - boxheight / 2;
       long yc = boxheight / 2;
-      const double RADIUS = boxheight * 0.4;
+      const double RADIUS = boxheight / 3;
       DrawEllipse(long(xc * l->scX), long(yc * l->scY),
 		  long(RADIUS * l->scX), long(RADIUS * l->scY));
 
@@ -919,13 +975,15 @@ svxPrintout::draw_info_box()
       DrawTo(bx, by);
 
       SetColour(PR_COLOUR_TEXT);
-      MOVEMM(div, boxheight - 5);
+      MOVEMM(div + 0.5, boxheight - 5.5);
       WriteString(wmsg(/*North*/115));
 
       wxString angle;
       angle.Printf(wxT("%03d"), l->rot);
       angle += wmsg(/*°*/344);
       wxString s;
+      /* TRANSLATORS: This is used on printouts of plans, with %s replaced by
+       * something like "123°".  The bearing is up the page. */
       s.Printf(wmsg(/*Plan view, %s up page*/168), angle.c_str());
       MOVEMM(2, 12); WriteString(s);
       break;
@@ -933,7 +991,7 @@ svxPrintout::draw_info_box()
     case layout::ELEV: case layout::TILT: {
       const int L = div + 2;
       const int R = boxwidth - 2;
-      const int H = boxheight / 2 - 5;
+      const int H = boxheight / 2;
       MOVEMM(L, H); DRAWMM(L + 5, H - 3); DRAWMM(L + 3, H); DRAWMM(L + 5, H + 3);
 
       DRAWMM(L, H); DRAWMM(R, H);
@@ -943,14 +1001,15 @@ svxPrintout::draw_info_box()
       MOVEMM((L + R) / 2, H - 2); DRAWMM((L + R) / 2, H + 2);
 
       SetColour(PR_COLOUR_TEXT);
-      MOVEMM(div, boxheight - 5);
+      MOVEMM(div + 2, boxheight - 8);
+      /* TRANSLATORS: "Elevation on" 020 <-> 200 degrees */
       WriteString(wmsg(/*Elevation on*/116));
       
-      MOVEMM(L, boxheight / 2);
+      MOVEMM(L, 2);
       WriteString(wxString::Format(wxT("%03d%s"),
 				   (l->rot + 270) % 360,
 				   wmsg(/*°*/344).c_str()));
-      MOVEMM(R - 10, boxheight / 2);
+      MOVEMM(R - 10, 2);
       WriteString(wxString::Format(wxT("%03d%s"),
 				   (l->rot + 90) % 360,
 				   wmsg(/*°*/344).c_str()));
@@ -960,11 +1019,18 @@ svxPrintout::draw_info_box()
       angle += wmsg(/*°*/344);
       wxString s;
       if (l->view == layout::ELEV) {
+	  /* TRANSLATORS: This is used on printouts of elevations, with %s
+	   * replaced by something like "123°".  The bearing is the direction
+	   * we’re looking. */
 	  s.Printf(wmsg(/*Elevation facing %s*/169), angle.c_str());
       } else {
 	  wxString a2;
 	  a2.Printf(wxT("%d"), l->tilt);
 	  a2 += wmsg(/*°*/344);
+	  /* TRANSLATORS: This is used on printouts of tilted elevations, with
+	   * the first %s replaced by something like "123°", and the second by
+	   * something like "-45°".  The bearing is the direction we’re
+	   * looking. */
 	  s.Printf(wmsg(/*Elevation facing %s, tilted %s*/284), angle.c_str(), a2.c_str());
       }
       MOVEMM(2, 12); WriteString(s);
@@ -973,6 +1039,7 @@ svxPrintout::draw_info_box()
     case layout::EXTELEV:
       SetColour(PR_COLOUR_TEXT);
       MOVEMM(2, 12);
+      /* TRANSLATORS: This is used on printouts of extended elevations. */
       WriteString(wmsg(/*Extended elevation*/191));
       break;
    }
@@ -1076,7 +1143,7 @@ svxPrintout::draw_scale_bar(double x, double y, double MaxLength)
 #endif
       buf.Printf(wxT("%d"), c * Step);
       SetColour(PR_COLOUR_TEXT);
-      MOVEMM(x + c * d - buf.length(), y - 4);
+      MOVEMM(x + c * d - buf.length(), y - 5);
       WriteString(buf);
    }
 }
@@ -1317,6 +1384,8 @@ svxPrintout::OnPrintPage(int pageNum) {
 	draw_info_box();
     }
 
+    pdc->SetClippingRegion(x_offset, y_offset,xpPageWidth+1, ypPageDepth+1);
+
     const double Sc = 1000 / l->Scale;
 
     if (l->show_mask & LEGS) {
@@ -1662,8 +1731,10 @@ svxPrintout::DrawEllipse(long x, long y, long r, long R)
     if (cur_pass != -1) {
 	x_t = x_offset + x - clip.x_min;
 	y_t = y_offset + clip.y_max - y;
+	const wxBrush & save_brush = pdc->GetBrush();
 	pdc->SetBrush(*wxTRANSPARENT_BRUSH);
 	pdc->DrawEllipse(x_t - r, y_t - R, 2 * r, 2 * R);
+	pdc->SetBrush(save_brush);
     }
 }
 
@@ -1686,10 +1757,10 @@ svxPrintout::Pre()
 			      false, wxString(fontname, wxConvUTF8),
 			      wxFONTENCODING_ISO8859_1);
     current_font = font_default;
-    pen_leg = new wxPen(colour_leg,0,wxSOLID);
-    pen_surface_leg = new wxPen(colour_surface_leg,0,wxSOLID);
-    pen_cross = new wxPen(colour_cross,0,wxSOLID);
-    pen_frame = new wxPen(colour_frame,0,wxSOLID);
+    pen_leg = new wxPen(colour_leg);
+    pen_surface_leg = new wxPen(colour_surface_leg);
+    pen_cross = new wxPen(colour_cross);
+    pen_frame = new wxPen(colour_frame);
     return 1; /* only need 1 pass */
 }
 
@@ -1716,7 +1787,6 @@ svxPrintout::NewPage(int pg, int pagesX, int pagesY)
 		  m_layout->datestamp.c_str());
     WriteString(footer);
     pdc->DestroyClippingRegion();
-    pdc->SetClippingRegion(x_offset, y_offset,xpPageWidth+1, ypPageDepth+1);
     drawticks(clip, (int)(9 * m_layout->scX / POINTS_PER_MM), x, y);
 }
 
