@@ -25,8 +25,11 @@
 
 #include "avenprcore.h"
 
-// The Gnome print dialog provides its own preview, as does Mac OS X.
-#if !defined __WXMAC__ && !defined wxUSE_LIBGNOMEPRINT
+// The libgnomeprint and GtkPrint dialogs provide their own preview, as does
+// Mac OS X.
+#if !(defined wxUSE_LIBGNOMEPRINT && wxUSE_LIBGNOMEPRINT) && \
+    !(defined wxUSE_GTKPRINT && wxUSE_GTKPRINT) && \
+    !defined __WXMAC__
 # define AVEN_PRINT_PREVIEW
 #endif
 
@@ -49,6 +52,7 @@ class svxPrintDlg : public wxDialog {
 //	wxCheckBox* m_blanks;
 	wxString m_File;
 	MainFrm* mainfrm;
+	bool close_after;
 
 	void LayoutToUI();
 	void UIToLayout();
@@ -56,12 +60,17 @@ class svxPrintDlg : public wxDialog {
 	void SomethingChanged(int control_id);
  public:
 	svxPrintDlg(MainFrm* parent, const wxString & filename,
-		    const wxString & title,
+		    const wxString & title, const wxString & cs_proj,
 		    const wxString & datestamp, time_t datestamp_numeric,
 		    double angle, double tilt_angle,
 		    bool labels, bool crosses, bool legs, bool surf,
 		    bool tubes, bool ents, bool fixes, bool exports,
-		    bool printing);
+		    bool printing, bool close_after_ = false);
+
+	~svxPrintDlg() {
+	    if (close_after) mainfrm->Close();
+	}
+
 	void OnPrint(wxCommandEvent& event);
 	void OnExport(wxCommandEvent& event);
 #ifdef AVEN_PRINT_PREVIEW
@@ -73,6 +82,7 @@ class svxPrintDlg : public wxDialog {
 	void OnElevationUpdate(wxUpdateUIEvent& e);
 	void OnChangeSpin(wxSpinEvent& event);
 	void OnChange(wxCommandEvent& event);
+	void OnCancel(wxCommandEvent& event);
  private:
 	DECLARE_EVENT_TABLE()
 };
